#include <iostream>
#include <ctime>
#include <algorithm>
#include "constantes.h"
#include "IA.h"

using namespace std;

IA::IA(): actual(0) {
  for (int i = 0; i < 3; i++) // El 3 variará en función al numero de inteligencias que hayamos creado
    inteligencias.push_back(i);
  
  srand(time(0));
  random_shuffle(inteligencias.begin(), inteligencias.end()); // Desordenamos el vector
}

IA::~IA() {}

// Esta función selecciona que sistema experto utilizará cada robot.

int IA::selectorIA(Soporte *s, Tablero *t, int numInteligencias) 
{
  int fin;  

  if (inteligencias[actual] == 0) 
    fin = roboti(s, t);

  else if (inteligencias[actual] == 1)
    fin = robotii(s, t);

  else
    fin = robotiii(s, t);

  if (actual < numInteligencias - 1)
    actual++;
  else
    actual = 0;

  return fin;
}

// Funcionamiento: Si ESTÁ abierto, intenta poner fichas sueltas, en caso de no conseguirlo intenta poner conjuntos y en caso de no conseguirlo intenta poner escaleras. Si NO ESTÁ abierto, intenta poner conjuntos, y si estos no suman más de 30 puntos intenta poner escaleras.

int IA::roboti(Soporte *s, Tablero *t)
{
  int estado;
  sCopia = *s;
  tCopia = *t; 

  if (sCopia.abierto() && !sCopia.vacio()) {
    estado = ponerFichas();
    if (estado == 0) {
      estado = ponerConjunto();
      if (estado == 0) {
	estado = ponerEscalera();
      }
    }
  }

  else if (!sCopia.vacio()){
    int suma = sCopia.sumaSoporte();
    estado = ponerConjunto();
    if (sCopia.sumaSoporte() > suma - 30 && !sCopia.vacio()) 
      estado = ponerEscalera();
  }

  *s = sCopia;
  *t = tCopia;

  if (sCopia.vacio())
    return 1;
  else 
    return 0;
}

// Funcionamiento: Si ESTÁ abierto y tiene más de 40 puntos o menos de 20 intenta quitarse el mayor número de fichas poniendo escaleras y conjuntos, si tiene entre 40 y 20 puntos intenta poner fichas sueltas. Si NO ESTÁ abierto intenta poner tanto escaleras como conjuntos. Este robot intenta quedarse rapidamente con pocas fichas y cuando tiene muy pocos puntos (20), intenta ganar poniendolo todo.

int IA::robotii(Soporte *s, Tablero *t)
{
  int estado;
  sCopia = *s;
  tCopia = *t; 

  if (sCopia.abierto() && !sCopia.vacio()) {
    if (sCopia.sumaSoporte() > 40 || sCopia.sumaSoporte() < 20) {
      estado = ponerConjunto();
      if (sCopia.sumaSoporte() > 40 || sCopia.sumaSoporte() < 20)
      estado = ponerEscalera();
      if (sCopia.sumaSoporte() > 40 || sCopia.sumaSoporte() < 20)
	estado = ponerFichas();
    }
  }

  else if (!sCopia.vacio()){
    estado = ponerConjunto();
    estado = ponerEscalera();
  }

  *s = sCopia;
  *t = tCopia;

  if (sCopia.vacio())
    return 1;
  else 
    return 0;
}

// Funcionamiento: Si ESTÁ abierto  y tiene más de 3 fichas, intenta poner conjuntos y escaleras y si tiene 3 o menos fichas intenta poner fichas sueltas. Si NO ESTÁ abierto intenta escaleras y si con ellas no suma más de 30 puntos intenta poner conjuntos. Este robot intenta quitarse rapidamente fichas y cuando solo tiene 3 juega con el tablero, de esta forma, no modifica los conjuntos del tablero hasta que no va a ganar.

int IA::robotiii(Soporte *s, Tablero *t)
{
  int estado;
  sCopia = *s;
  tCopia = *t; 

  if (sCopia.abierto() && !sCopia.vacio()) {
    if (sCopia.fichas() > 3) {
      estado = ponerConjunto();
      estado = ponerEscalera();
    }
     
    else 
      estado = ponerFichas();
  }

  else if (!sCopia.vacio()){
    int suma = sCopia.sumaSoporte();
    estado = ponerEscalera();
    if (sCopia.sumaSoporte() > suma - 30 && !sCopia.vacio()) 
      estado = ponerConjunto();
  }

  *s = sCopia;
  *t = tCopia;

  if (sCopia.vacio())
    return 1;
  else 
    return 0;
}

int IA::ponerConjunto()
{
  Ficha f;
  bool tieneComodin = false, opValida = false;
  int suma = sCopia.sumaSoporte();

  //int p;// A BORRAR

  sCopia.ordenarNumero();
  
  // Comprobamos si tiene comodin
  f = sCopia.escogerFicha(0); // El comodín estará en la primera posición si es que hay.
  if (f.comodin())
    tieneComodin = true;
  sCopia.ordenar(f, 0); // Volvemos a colocar la ficha que fuera una vez usada.

  int i = sCopia.fichas() - 1;

  //cout << "Despues de comprobar si tiene comodin" << endl;
  //cout << "Valor de i: " << i << endl << "Tiene comodin: " << tieneComodin << endl;

  // Empezamos a comprobar fichas de la última a la primera, para insertar primero las de mayor puntuación
  while (i >= 0) {
    f = sCopia.escogerFicha(i);    


    //cout << "\n\nescogida ficha: " << f.numero() << " " << f.color() << endl;
    //cout << "Valor de i: " << i << endl << "Tiene comodin: " << tieneComodin << endl;
    //cout << "fichas en el soporte: " << sCopia.fichas() << endl;
    /*if (conj.empty())
      cout << "conj está vacio" << endl;
    else {
      for (unsigned r = 0; r < conj.size(); r++)
	cout << "conj[" << r << "] = " << conj[r].numero() << " " << conj[r].color() << endl;
    }
    cout << endl;*/

    opValida = comprobarFicha(f); // Metemos ficha.
    
    if (!opValida) { // Si la ficha no corresponde con las que ya se han procesado.
      sCopia.ordenar(f, i); // la dejamos donde estaba

      if (tieneComodin && conj.size() < 3) { // y si tiene comodin y el conjunto no es válido, probamos con él
	//cout << "OP INVALIDA Entro en tieneComodin && conj.size() < 3" << endl;
	f = sCopia.escogerFicha(0);
	f.modificarComodin(conj[0].numero(), conj[0].color());
	opValida = comprobarFicha(f);
	tieneComodin = false; // una vez usado ya no tenemos comodín
	i--;
      }

      else if (conj.size() >= 3) { // Si no, si el conjunto tiene 3 o más fichas, las insertamos en el tablero
	//cout << "OP INVALIDA Entro en conj.size() >=3" << endl;
	insertarConjunto(); // Insertamos el conjunto
	while (!conj.empty()) // y lo eliminamos del auxiliar
	  conj.erase(conj.begin());

	sCopia.ordenarNumero(); // y por último volvemos a reagrupar.
      }

      else { // Si el conjunto no es válido ni se tiene comodín, pues se insertan de nuevo las fichas.
	//cout << "OP INVALIDA Entro en else" << endl;
	sCopia.ordenarNumero();
	int k = sCopia.fichas();
	while (!conj.empty()) {
	  if (conj[0].comodin()) { // Si se a usado el comodín
	    i++; // incrementamos el indice para no saltarnos ninguna comprobación
	    tieneComodin = true;
	  }
	  sCopia.ordenar(conj[0], k); // Insertamos las fichas del conjunto auxiliar al final del soporte
	  conj.erase(conj.begin()); // eliminamos la ficha procesada.
	  k++;
	}
	sCopia.ordenarNumero(); // Y dejamos el soporte como estaba
      }
    }

    else if (i == 0 && conj.size() >= 3) { // Si la operación es valida, el conjunto tiene 3 fichas y la i vale 0, insertamos el conjunto
      //cout << "OP VALIDA Entro en i==0 && conj.size() >= 3" << endl;
      insertarConjunto(); // Insertamos el conjunto
      while (!conj.empty()) // y lo eliminamos del auxiliar
	conj.erase(conj.begin());

      sCopia.ordenarNumero(); // y por último volvemos a reagrupar.
      i--;
    }

    else if (i == 0 && conj.size() < 3) { // Si la operación es valida, la i vale 0, pero el conjunto no tiene 3 o más fichas, pues las dejamos en el soporte 
      //cout << "OP VALIDA Entro en i==0 && conj.size() < 3" << endl;
      sCopia.ordenarNumero();
      int k = sCopia.fichas();
      while (!conj.empty()) {
	sCopia.ordenar(conj[0], k); // Insertamos las fichas del conjunto auxiliar al final del soporte
	conj.erase(conj.begin()); // eliminamos la ficha procesada.
	k++;
      }
      sCopia.ordenarNumero();
      i--;
    }

    else { // si no, pues comprobamos con la siguiente ficha.
      //cout << "OP VALIDA Entro en else i--" << endl;
      i--;
    }      

    //cin >> p;
  }

  while (!conj.empty())
    conj.erase(conj.begin());

  if (suma > sCopia.sumaSoporte()) // Si ha puesto alguna ficha
    return 1;
  else 
    return 0;
}

int IA::ponerEscalera()
{
  Ficha f, com1, com2;
  int contCom = 0;
  bool opValida = false;
  int suma = sCopia.sumaSoporte();
  //int p;// A BORRAR

  sCopia.ordenarColor();

  com1 = sCopia.escogerFicha(0); // Escogemos la primera ficha para ver si es comodin
  contCom++; // Incrementamos el contador de comodines
  if (com1.comodin()) { // Si es comodin
    com2 = sCopia.escogerFicha(1); // miramos si tiene el otro comodin
    contCom++; // incrementamos nuevamente el contador

    if (!com2.comodin()) { // si no es comodin 
      sCopia.ordenar(com2, 1); // dejamos la ficha de donde la cogimos
      contCom--; // decrementamos el contador
      com2 = Ficha(false, 0); // y dejamos inutilizable la ficha
    }
  }
  else { // Si es comodin
    sCopia.ordenar(com1, 0); // Dejamos la ficha donde estaba
    contCom--; // decrementamos el contador
    com1 = Ficha(false, 0); // y dejamos la ficha inutilizable
  }

  sCopia.ordenarColor(); // ordenamos nuevamente sin comodines 

  int i = sCopia.fichas() - 1;

  // Empezamos a comprobar fichas de la última a la primera, para insertar primero las de mayor puntuación
  while (i >= 0) {
    f = sCopia.escogerFicha(i);

    opValida = comprobarFicha(f); // Metemos ficha.

    if (!opValida) { // Si la ficha no corresponde con las que ya se han procesado.
      sCopia.ordenar(f, i); // la dejamos donde estaba

      if (contCom > 0 && conj.size() < 3) { // y si tiene comodin y el conjunto no es válido, probamos con él

	if (contCom == 2) {

	  if (conj[0].numero() == 1)
	    com2.modificarComodin(conj[conj.size()-1].numero() + 1, conj[0].color());

	  else
	    com2.modificarComodin(conj[0].numero() - 1, conj[0].color());

	  opValida = comprobarFicha(com2);
	}

	else if (contCom == 1) {
	  
	  if (conj[0].numero() == 1)
	    com1.modificarComodin(conj[conj.size()-1].numero() + 1, conj[0].color());

	  else
	    com1.modificarComodin(conj[0].numero() - 1, conj[0].color());

	  opValida = comprobarFicha(com1);
	}

	if (opValida) {
	  contCom--;
	  f = sCopia.escogerFicha(i);
	  opValida = comprobarFicha(f);
	  
	  if (!opValida)
	    sCopia.ordenar(f, i); // la dejamos donde estaba
	}
	i--;
      }

      else if (conj.size() >= 3) { // Si no, si el conjunto tiene 3 o más fichas, las insertamos en el tablero
	insertarConjunto(); // Insertamos el conjunto
	while (!conj.empty()) // y lo eliminamos del auxiliar
	  conj.erase(conj.begin());

	sCopia.ordenarColor(); // y por último volvemos a reagrupar.
      }

      else { // Si el conjunto no es válido ni se tiene comodín, pues se insertan de nuevo las fichas.

	sCopia.ordenarColor();
	int k = sCopia.fichas();
	while (!conj.empty()) {
	  if (conj[0].comodin()) {// Si se a usado el comodín
	    contCom++;
	    i++;
	  }
	  else
	    sCopia.ordenar(conj[0], k); // Insertamos las fichas del conjunto auxiliar al final del soporte
	  conj.erase(conj.begin()); // eliminamos la ficha procesada.
	  k++;
	}
	sCopia.ordenarColor(); // Y dejamos el soporte como estaba
      } 
    }

    else if (i == 0 && conj.size() >= 3) { // Si la operación es valida, el conjunto tiene 3 fichas y la i vale 0, insertamos el conjunto

      insertarConjunto(); // Insertamos el conjunto
      while (!conj.empty()) // y lo eliminamos del auxiliar
	conj.erase(conj.begin());

      sCopia.ordenarColor(); // y por último volvemos a reagrupar.
      i--;
    }

    else if (i == 0 && conj.size() < 3) { // Si la operación es valida, la i vale 0, pero el conjunto no tiene 3 o más fichas, pues las dejamos en el soporte 
      
      sCopia.ordenarColor();
      int k = sCopia.fichas();
      while (!conj.empty()) {
	if (!conj[0].comodin())
	  sCopia.ordenar(conj[0], k); // Insertamos las fichas del conjunto auxiliar al final del soporte
	conj.erase(conj.begin()); // eliminamos la ficha procesada.
	k++;
      }
      sCopia.ordenarColor();
      i--;
    }

    else // si no, pues comprobamos con la siguiente ficha.
      i--;
  }

  sCopia.ordenarColor();

  if (contCom == 2) {
    sCopia.ordenar(com2, sCopia.fichas());
    sCopia.ordenar(com1, sCopia.fichas());  
  }

  else if (contCom == 1)
    sCopia.ordenar(com1, sCopia.fichas());

  sCopia.ordenarColor();

  while (!conj.empty())
    conj.erase(conj.begin());

  if (suma > sCopia.sumaSoporte())
    return 1;
  else 
    return 0;
}

int IA::ponerFichas()
{
  Ficha f, com1, com2;
  int suma = sCopia.sumaSoporte(), i = 0, j = 0, k = 0, contCom = 0;

  sCopia.ordenarColor();

  com1 = sCopia.escogerFicha(0); // Escogemos la primera ficha para ver si es comodin
  contCom++; // Incrementamos el contador de comodines
  if (com1.comodin()) { // Si es comodin
    com2 = sCopia.escogerFicha(1); // miramos si tiene el otro comodin
    contCom++; // incrementamos nuevamente el contador

    if (!com2.comodin()) { // si no es comodin 
      sCopia.ordenar(com2, 1); // dejamos la ficha de donde la cogimos
      contCom--; // decrementamos el contador
      com2 = Ficha(false, 0); // y dejamos inutilizable la ficha
    }
  }
  else { // Si es comodin
    sCopia.ordenar(com1, 0); // Dejamos la ficha donde estaba
    contCom--; // decrementamos el contador
    com1 = Ficha(false, 0); // y dejamos la ficha inutilizable
  }

  sCopia.ordenarColor();

  while (i < FILAS) {
    while (j < COLUMNAS) {
      f = tCopia.escogerFicha(i, j); // Vamos comprobando posiciones

      if (!f.fichaNula())
	conj.push_back(f);

      else if (!conj.empty()) {
	k = 0;

	while (k < sCopia.fichas()) {
	  sCopia.ordenarColor();
	  f = sCopia.escogerFicha(k);
	 
	  if (f.numero() == conj[0].numero() - 1 && f.color() == conj[0].color() && f.color() == conj[conj.size()-1].color()) {
	    conj.insert(conj.begin(), f);
	    if (k > 0) 
	      k--;
	  }
	  
	  else if (f.numero() == conj[conj.size()-1].numero() + 1 && f.color() == conj[0].color() && f.color() == conj[conj.size()-1].color()) {
	    conj.push_back(f);
	    if (k > 0)
	      k--;
	  }
	  
	  else if (f.numero() == conj[0].numero() && f.numero() == conj[conj.size()-1].numero() && !colorRepetido(f)) {
	    conj.push_back(f);
	    if (k > 0)
	      k--;
	  }
	  
	  else {
	    sCopia.ordenar(f, k);
	    k++;
	  }	    
	}

	insertarConjunto();
	while (!conj.empty()) // y lo eliminamos del auxiliar
	  conj.erase(conj.begin());
      }
      j++;
    }
    i++;
    j = 0;
  }

  if (contCom == 2) {
    sCopia.ordenar(com2, sCopia.fichas());
    sCopia.ordenar(com1, sCopia.fichas());  
  }
  
  else if (contCom == 1)
    sCopia.ordenar(com1, sCopia.fichas());
  
  sCopia.ordenarColor();
  
  while (!conj.empty())
    conj.erase(conj.begin());
  
  if (suma > sCopia.sumaSoporte())
    return 1;
  else 
    return 0;
}

bool IA::comprobarFicha(const Ficha& f)
{
  bool opValida = false;
  // Si el conjunto está vacio, se puede insertar directamente una ficha en el, sin importar que ficha sea.
  if (conj.empty()) {
    conj.push_back(f);
    opValida = true;
  }

  // Si hay una sola ficha tenemos que comprobar si es valida con la que ya está
  else if (conj.size() == 1) {
    Ficha primera = conj[0];

    // Si las dos son comodines, es inválido.
    if (f.comodin() && primera.comodin()) 
      cout << "operación invalida" << endl;

    // Si alguna de las dos es comodín, se inserta directamente.
    else if (f.comodin() || primera.comodin()) {
      if (f.numero() <= primera.numero())
	conj.insert(conj.begin(), f);
      else
	conj.push_back(f);
      opValida = true;
    }

      // Si no, si tienen igual numero y color distinto o si tienen igual color y la diferencia entre los numeros es 1
    else if ((f.numero() == primera.numero() && f.color() != primera.color()) || (f.color() == primera.color() && (f.numero()== primera.numero() + 1))) {
      conj.push_back(f);
      opValida = true;
    }

    else if ((f.color() == primera.color() && (f.numero()== primera.numero() - 1))) {
      conj.insert(conj.begin(), f);
      opValida = true;
    }
  }

  // Si hay más de una ficha hay que comprobar que la ficha corresponde con las ya insertadas.
  else if (conj.size() > 1) {
    Ficha primera = conj[0];
    Ficha ultima = conj[conj.size() - 1];

    if ((f.comodin() && ultima.comodin()) || (f.comodin() && primera.comodin())) 
      cout << "";

    else if (f.numero() == primera.numero() && f.numero() == ultima.numero() && !colorRepetido(f)) {
	conj.push_back(f);
	opValida = true;
    }

    else if (f.color() == primera.color() && f.color() == ultima.color() && (f.numero()== ultima.numero() + 1)) {  
        conj.push_back(f);
        opValida = true;
    }

    else if (f.color() == primera.color() && f.color() == ultima.color() && (f.numero()== primera.numero() - 1)) {  
      conj.insert(conj.begin(), f);
        opValida = true;
    }
  }

  return opValida;
}

bool IA::colorRepetido(const Ficha& f)
{
  bool bandera = false;

  if (conj.size() >= 4)
    bandera = true; 

  else if (!f.comodin()) {
    for (unsigned i = 0; i < conj.size(); i++)
      if (conj[i].color() == f.color())
	bandera = true;
  }

  return bandera;
}

void IA::insertarConjunto()
{
  int tamConj = conj.size(), cont, i, j;
  unsigned k;
  Ficha f;
  bool bandera = false;

  if (tamConj >= 3) { // comprobamos que el tamaño del conjunto sea adecuado.
    // Tenemos que buscar suficientes posiciones vacias para poner el conjunto
    cont = 0;
    i = 0;
    j = 0;

    // Recorremos el tablero en busca de posiciones vacias
    while (!bandera && i < FILAS) {
      while (!bandera && j < COLUMNAS) {
	f = tCopia.escogerFicha(i, j); // Vamos comprobando posiciones
	if (f.fichaNula()) // Si en una casilla no hay ficha, incrementamos el contador
	  cont++;
	else { // Si no dejamos la ficha donde estaba y ponemos el contador a 0 de nuevo.
	  tCopia.insertarFicha(f, i, j);
	  cont = 0;
	}

	if (cont == (tamConj + 2)) // Si hemos encontrado tantas posiciones vacias como fichas tiene el conjunto, más dos posiciones más para dejar vacias antes y después del mismo, pues salimos del bucle e insertamos en dichas posiciones. 
	  bandera = true;
	
	else // Si no incrementamos la posicion 
	  j++;
      }
      if (!bandera) {
	i++;
	cont = 0;
	j = 0;
      }
    }

    if (bandera) { // Si hemos encontrado un sitio donde poner el conjunto
      // Modificamos j, para que esté en la posición que queremos que empiece a poner
      j -= tamConj;
      k = 0; // indice del conjunto
      while (k < conj.size()) {
	tCopia.insertarFicha(conj[k], i, j); // Insertamos la ficha
	k++; // incrementamos indices
	j++;	
      }
    }
  }
}
