#ifndef _IA_H_
#define _IA_H_

#include <vector>
#include "soporte.h"
#include "jugador.h"
#include "tablero.h"

using namespace std;

/** Clase IA. Clase para gestionar la inteligencia artificial y los sistemas expertos del juego.
*
* Distribuido bajo licencia GNU GPLv.3
*
* @author Daniel Chaves Vázquez
* @version 1.0
*/
class IA {
 public:

  // Constructor

  /** Constructor de la clase IA.
   */
  IA();

  /** Método que selecciona aleatoriamente un robot (sistema experto) para cada una de las IA.
   * @param s Puntero a un objeto de la clase Soporte.
   * @param t Puntero a un objeto de la clase Tablero.
   * @param numInteligencias Indica el número de IA que hay en la partida.
   * @return Devuelve 0 si aún quedan fichas en el soporte (el juego continua) o 1 si está vacio (el robot ha ganado).
   */
  int selectorIA(Soporte *s, Tablero *t, int numInteligencias);

  /** Sistema experto número 1: 
   * Funcionamiento: Si ESTÁ abierto, intenta poner fichas sueltas, en caso de no conseguirlo intenta poner conjuntos y en caso de no conseguirlo intenta poner escaleras. Si NO ESTÁ abierto, intenta poner conjuntos, y si estos no suman más de 30 puntos intenta poner escaleras.
   * @param s Puntero a un objeto de la clase Soporte.
   * @param t Puntero a un objeto de la clase Tablero.
   * @return Devuelve 0 si aún quedan fichas en el soporte (el juego continua) o 1 si está vacio (el robot ha ganado).
   */
  int roboti(Soporte *s, Tablero *t);  

  /** Sistema experto número 2: 
   * Funcionamiento: Si ESTÁ abierto y tiene más de 40 puntos o menos de 20 intenta quitarse el mayor número de fichas poniendo escaleras y conjuntos, si tiene entre 40 y 20 puntos intenta poner fichas sueltas. Si NO ESTÁ abierto intenta poner tanto escaleras como conjuntos. Este robot intenta quedarse rapidamente con pocas fichas y cuando tiene muy pocos puntos (20), intenta ganar poniendolo todo.
   * @param s Puntero a un objeto de la clase Soporte.
   * @param t Puntero a un objeto de la clase Tablero.
   * @return Devuelve 0 si aún quedan fichas en el soporte (el juego continua) o 1 si está vacio (el robot ha ganado).
   */
  int robotii(Soporte *s, Tablero *t);

  /** Sistema experto número 3:
   * Funcionamiento: Si ESTÁ abierto  y tiene más de 3 fichas, intenta poner conjuntos y escaleras y si tiene 3 o menos fichas intenta poner fichas sueltas. Si NO ESTÁ abierto intenta escaleras y si con ellas no suma más de 30 puntos intenta poner conjuntos. Este robot intenta quitarse rapidamente fichas y cuando solo tiene 3 juega con el tablero, de esta forma, no modifica los conjuntos del tablero hasta que no va a ganar.
   * @param s Puntero a un objeto de la clase Soporte.
   * @param t Puntero a un objeto de la clase Tablero.
   * @return Devuelve 0 si aún quedan fichas en el soporte (el juego continua) o 1 si está vacio (el robot ha ganado).
   */
  int robotiii(Soporte *s, Tablero *t);

  /** Destructor de la clase IA.
   */
  ~IA();

 private:
  Soporte sCopia;
  Tablero tCopia;
  vector <int> inteligencias;
  typedef vector<Ficha> Conjunto;
  Conjunto conj;
  int actual;

  /** Método encargado de poner conjuntos (mismo número, distinto color) en el tablero. 
   * @return Devuelve 1 si se ha puesto alguna ficha y 0 en caso contrario.
   */
  int ponerConjunto();

  /** Método encargado de poner escaleras (mismo color, numéros correlativos ascendentemente) en el tablero. 
   * @return Devuelve 1 si se ha puesto alguna ficha y 0 en caso contrario.
   */
  int ponerEscalera();

  /** Método encargado de poner fichas sueltas en el tablero. 
   * @return Devuelve 1 si se ha puesto alguna ficha y 0 en caso contrario.
   */
  int ponerFichas();

  /** Método auxiliar de ponerConjunto, ponerEscalera y ponerFichas, que comprueba si una ficha es válida con las demás que se estén comprobando.
   * @param f Referencia constante a un objeto de la clase Ficha.
   * @return Devuelve si la ficha es válida con las demás que se están comprobando o no.
   */
  bool comprobarFicha(const Ficha& f);

  /** Método auxiliar de comprobarFicha, que comprueba si para un conjunto, el color está repetido o no.
   * @param f Referencia constante a un objeto de la clase Ficha.
   * @return Devuelve si el color de la ficha está repetido en el conjunto que se está comprobando.
   */
  bool colorRepetido(const Ficha& f);

  /** Método que inserta el conjunto ya comprobado en el tablero.
   */
  void insertarConjunto();
};

#endif
