#ifndef _FUENTE_H_
#define _FUENTE_H_

#include <SDL/SDL.h>
#include <SDL/SDL_ttf.h>

/** Clase Fuente. Clase para gestionar el trabajo con la libreria ttf.
*
* Distribuido bajo licencia GNU GPLv.3
*
* @author Daniel Chaves Vázquez
* @version 1.0
*/
class Fuente
{
 public:

  // Constructor

  /** Constructor de la clase Fuente.
   * @param ruta Ruta del fichero donde se encuentra la fuente que se va a cargar.
   * @param tam Indica el tamaño de la fuente.
   */
  Fuente(const char* ruta, int tam);

  // Modificadores

  /** Método para dar a la fuente el estilo negrita (bold).
   */
  void negrita();
  
  /** Método para dar a la fuente el estilo cursiva (italic).
   */
  void cursiva();

  /** Método para dar a la fuente el estilo subrayado (underlined).
   */
  void subrayado();

  /** Método para dar a la fuente el estilo normal.
   */
  void normal();

  // Observadores

  /** Método para manejar cómo se mostrarán los textos por pantalla.
   * @param v Puntero a un objeto de la clase que contiene la superficie donde se va a dibujar la fuente.
   * @param frase Cadena de caracteres que contiene la frase que se desea mostrar.
   * @param x Indica la coordenada en el eje x.
   * @param y Indica la coordenada en el eje y.
   * @param r Indica la componente r de RGB para definir el color.
   * @param g Indica la componente g de RGB para definir el color.
   * @param b Indica la componente b de RGB para definir el color.
   */
  void mostrarTexto(Video *v, const char* frase, int x, int y, int r = 0, int g = 0, int b = 0);

  /** Destructor de la clase Fuente.
   */
  ~Fuente();
 private:
  TTF_Font *fuente;
  SDL_Surface *texto;
  SDL_Color color;
  SDL_Rect posicion;
};

inline Fuente::Fuente(const char* ruta, int tam) { fuente = TTF_OpenFont(ruta, tam); }
inline void Fuente::negrita() { TTF_SetFontStyle(fuente, TTF_STYLE_BOLD); }
inline void Fuente::cursiva() { TTF_SetFontStyle(fuente, TTF_STYLE_ITALIC); }
inline void Fuente::subrayado() { TTF_SetFontStyle(fuente, TTF_STYLE_UNDERLINE); }
inline void Fuente::normal() { TTF_SetFontStyle(fuente, TTF_STYLE_NORMAL); }
inline Fuente::~Fuente() {}

#endif
