#ifndef _GESTORJUGADORES_H_
#define _GESTORJUGADORES_H_

#include <vector>
#include <string>
#include "video.h"
#include "imagen.h"
#include "sonido.h"
#include "jugador.h"
#include "fuente.h"
#include "gestorPartida.h"

using namespace std;

/** Clase GestorJugadores. Clase para gestionar los menús referentes a los jugadores.
*
* Distribuido bajo licencia GNU GPLv.3
*
* @author Daniel Chaves Vázquez
* @version 1.0
*/
class GestorJugadores
{
 public:

  // Constructor

  /** Constructor de la clase GestorJugadores.
   */
  GestorJugadores();

  // Modificadores

  /** Método para ver las estadísticas de los jugadores que haya en el sistema.
   * @param v Puntero a un objeto de la clase que contiene la superficie donde se van a dibujar las imagenes.
   */
  void ver(Video *v);

  /** Método para borrar los jugadores que haya en el sistema.
   * @param v Puntero a un objeto de la clase que contiene la superficie donde se van a dibujar las imagenes.
   */
  void borrar(Video *v);

  /** Destructor de la clase GestorJugadores.
   */
  ~GestorJugadores();

 private:
  Imagen *fondo, *inc, *dec, *suena, *nosuena, *confirmar, *cancelar;
  typedef vector <string> Jugadores;
  Jugadores jugadores;
  Jugador *j;
  Fuente textos;
  string nombre;

  // Metodos privados

  /** Método que carga en memoria el nombre de todos los jugadores que haya en el sistema.
   */
  void cargarNombres();

  /** Método que muestra la pantalla cuando no hay jugadores en el sistema.
   * @param v Puntero a un objeto de la clase que contiene la superficie donde se van a dibujar las imagenes. 
   */
  void pantallaNo(Video *v);

  /** Método que muestra la pantalla para ver las estadísticas de los jugadores.
   * @param v Puntero a un objeto de la clase que contiene la superficie donde se van a dibujar las imagenes. 
   */
  void actualizarPantalla(Video *v);

  /** Método que muestra la pantalla para borrar los jugadores.
   * @param v Puntero a un objeto de la clase que contiene la superficie donde se van a dibujar las imagenes. 
   */
  void actualizarBorrar(Video * v);

  /** Método que devuelve si las coordenadas de un puntero están dentro del area de la flecha de incremento o no.
   * @param p Referencia constante a un objeto de la clase Puntero.
   * @return Devuelve si las coordenadas del puntero están o no en el area de la flecha de incremento.
   */
  bool areaIncremento(const Puntero& p);

  /** Método que devuelve si las coordenadas de un puntero están dentro del area de la flecha de decremento o no.
   * @param p Referencia constante a un objeto de la clase Puntero.
   * @return Devuelve si las coordenadas del puntero están o no en el area de la flecha de decremento.
   */
  bool areaDecremento(const Puntero& p);
};

inline GestorJugadores::~GestorJugadores() {}

#endif
