#include <iostream>
#include <algorithm>
#include <sstream>
#include "constantes.h"
#include "gestorPartida.h"

extern bool silencio;

using namespace std;

GestorPartida::GestorPartida(): textos("./fuentes/IMPACT.TTF", 30), ronda("./fuentes/IMPACT.TTF", 100), musica("sonidos/menu.wav"), numJugadores(3) /* mínimo 2 // máximo 4 */
{
  fondo = new Imagen("./imagenes/menus/fondoMenu.png");
  incrementar = new Imagen("./imagenes/menus/incrementar.png", INC_X, INC_Y);
  decrementar = new Imagen("./imagenes/menus/decrementar.png", DEC_X, DEC_Y);

  // Altavoz cancelacion de sonido
  suena = new Imagen("./imagenes/sonando.png", 0, 0);
  nosuena = new Imagen("./imagenes/silencio.png", 0 , 0);

  // Confirmar-Cancelar
  confirmar = new Imagen("./imagenes/confirmar.png", CONFIRMAR_X, CONFIRMAR_Y);
  cancelar = new Imagen("./imagenes/cancelar.png", CANCELAR_X, CANCELAR_Y);
}

void GestorPartida::partidaSimple(Video *v)
{
  musica.reproducir();

  seleccionJugadorSimple(v);

  if (jugadores.size() == numJugadores) {

    // Creamos la partida
    partida = new Partida(numJugadores);
    
    // Añadimos todos los jugadores a la partida
    for (unsigned i = 0; i < numJugadores; i++)
      partida->anadirJugador(jugadores[i]);
    
  // Jugamos
    partida->jugar(v);
    
    // Borramos la partida y los jugadores
    delete partida;
  }
  while (!jugadores.empty()) {
    jugadores.erase(jugadores.begin());
  }
}

void GestorPartida::concurso(Video *v)
{
  musica.reproducir();

  seleccionJugadores(v);

  if (jugadores.size() == numJugadores) {

    desordenarJugadores();

    for (unsigned i = 0; i < numJugadores; i++) {
      // Creamos la partida
      partida = new Partida(numJugadores);
      
      // Añadimos los jugadores
      for (unsigned j = 0; j < numJugadores; j++)
	partida->anadirJugador(jugadores[j]);
      
      pantallaRonda(i+1, v);

      // Jugamos
      partida->jugar(v, 1);
      
    // Eliminamos los jugadores
      for (unsigned k = 0; k < numJugadores; k++)
	partida->eliminarJugador(jugadores[k]);
      
      // Eliminamos la partida
      delete partida;

      // Modificamos los jugadores para que en la siguiente partida empiece el siguiente jugador
      incrementarPosicion();
    }
  }
  while (!jugadores.empty())
    jugadores.erase(jugadores.begin());
}

void GestorPartida::seleccionJugadores(Video* v)
{
  unsigned cont = 0;
  int opc = 0;
  SDL_Event evento;
  Puntero p;

  numJugadores = 3;

  nombre = "";

  // Bucle de seleccion del número de jugadores
  while (opc == 0) {
    actualizarPantalla(v, cont, 1, nombre);
    while (SDL_PollEvent(&evento)) {    
      if (evento.type == SDL_QUIT)
	opc = -1; // Salir  
      
      else if (evento.type == SDL_MOUSEBUTTONDOWN) {
	p.actualizar();
	
	if (p.boton() == 1 && areaAltavoz(p)) {
	  musica.silenciar();
	}

	else if (p.boton() == 1 && areaConfirmar(p)) {
	  opc = 1;
	}

	else if (p.boton() == 1 && areaCancelar(p)) {
	  opc = -1;
	}

	else if (p.boton() == 1 && areaIncremento(p)) {
	  if (numJugadores < 4)
	    numJugadores++;
	}
	else if (p.boton() == 1 && areaDecremento(p)) {
	  if (numJugadores > 2)
	    numJugadores--;
	}
      }
    }
    SDL_Delay(1);
  }

  while(!jugadores.empty())
    jugadores.erase(jugadores.begin());

  // Bucle seleccion de jugadores
  while (opc > 0 && cont < numJugadores) {
    actualizarPantalla(v, cont, 2, nombre);
    while (SDL_PollEvent(&evento)) {    
      if (evento.type == SDL_QUIT)
	opc = -1; // Salir  
      
      else if (evento.type == SDL_KEYDOWN) {

	if (evento.key.keysym.sym == SDLK_BACKSPACE) {
	  if (!nombre.empty())
	    nombre.erase(nombre.end()-1);
	}

	else if (nombre.length() < 8 && caracter(evento) != '+')
	    nombre.push_back(caracter(evento));
      }

      else  if (evento.type == SDL_MOUSEBUTTONDOWN) {
	p.actualizar();
	
	if (p.boton() == 1 && areaAltavoz(p)) {
	  musica.silenciar();
	}

	else if (p.boton() == 1 && areaConfirmar(p)) {
	  if (!reservado(nombre) && nombre != "") {
	    jugadores.push_back(new Jugador(nombre));
	    cont++;
	  }
	  nombre = "";
	}

	else if (p.boton() == 1 && areaCancelar(p)) {
	  opc = -1;
	}
      }
    }
    SDL_Delay(1);
  }
}

void GestorPartida::seleccionJugadorSimple(Video *v)
{
  int opc = 0;
  unsigned cont = 0;
  SDL_Event evento;
  Puntero p;

  nombre = "";

  // Bucle de seleccion del número de jugadores
  while (opc == 0) {
    actualizarPantalla(v, cont, 1, nombre);
    while (SDL_PollEvent(&evento)) {    
      if (evento.type == SDL_QUIT)
	opc = -1; // Salir  
      
      else if (evento.type == SDL_MOUSEBUTTONDOWN) {
	p.actualizar();

	if (p.boton() == 1 && areaAltavoz(p)) {
	  musica.silenciar();
	}
	
	else if (p.boton() == 1 && areaConfirmar(p)) {
	  opc = 1;
	}

	else if (p.boton() == 1 && areaCancelar(p)) {
	  opc = -1;
	}

	else if (p.boton() == 1 && areaIncremento(p)) {
	  if (numJugadores < 4)
	    numJugadores++;
	}
	else if (p.boton() == 1 && areaDecremento(p)) {
	  if (numJugadores > 2)
	    numJugadores--;
	}
      }
    }
    SDL_Delay(1);
  }

  while(!jugadores.empty())
    jugadores.erase(jugadores.begin());

  // Bucle seleccion de jugadores
  while (opc > 0) {
    actualizarPantalla(v, cont, 2, nombre);
    while (SDL_PollEvent(&evento)) {    
      if (evento.type == SDL_QUIT)
	opc = -1; // Salir  
      
      else if (evento.type == SDL_KEYDOWN) {
	
	if (evento.key.keysym.sym == SDLK_BACKSPACE) {
	  if (!nombre.empty())
	    nombre.erase(nombre.end()-1);
	}

	else if (nombre.length() < 8 && caracter(evento) != '+')
	    nombre.push_back(caracter(evento));
      }

      else if (evento.type == SDL_MOUSEBUTTONDOWN) {
	p.actualizar();
	
	if (p.boton() == 1 && areaAltavoz(p)) {
	  musica.silenciar();
	}

	else if (p.boton() == 1 && areaConfirmar(p)) {
	  if (!reservado(nombre) && nombre != "") {
	    jugadores.push_back(new Jugador(nombre));
	    opc = -1;
	  }
	  nombre = "";
	}

	else if (p.boton() == 1 && areaCancelar(p)) {
	  opc = -1;
	}
      }
    }
    SDL_Delay(1);
  }

  switch (numJugadores) {
  case 2:
    jugadores.push_back(new Jugador("robotuno", true));
    break;
  case 3: 
    jugadores.push_back(new Jugador("robotuno", true));
    jugadores.push_back(new Jugador("robotdos", true));
    break;
  case 4:
    jugadores.push_back(new Jugador("robotuno", true));
    jugadores.push_back(new Jugador("robotdos", true));
    jugadores.push_back(new Jugador("robotres", true));
    break;
  default: cout << "error" << endl;
  }
}

void GestorPartida::actualizarPantalla(Video *v, unsigned jugActual, int zona, string nom)
{
  stringstream datos;
  
  fondo->dibujar(v);

  confirmar->dibujar(v);
  cancelar->dibujar(v);

  if (!silencio)
    suena->dibujar(v);
  else
    nosuena->dibujar(v);

  datos.str("");
  datos << "Numero de jugadores:";
  textos.mostrarTexto(v, datos.str().c_str(), 64, 92, 255, 255, 255);
  datos.str("");
  datos << numJugadores;
  textos.mostrarTexto(v, datos.str().c_str(), 415, 92, 255, 255, 255);
  
  if (zona == 1) {
  incrementar->dibujar(v);
  decrementar->dibujar(v);
  datos.str("");
  datos << "(Pulse confirmar para continuar)";
  textos.mostrarTexto(v, datos.str().c_str(), 64, 140, 255, 255, 255);
  }

  else {
    datos.str("");
    datos << "Nombre del jugador " << jugActual+1 << " (8 car. max):";
    textos.mostrarTexto(v, datos.str().c_str(), 64, 240, 255, 255, 255);

    datos.str("");
    datos << nombre.c_str();
    if (datos.str() != "")
      textos.mostrarTexto(v, datos.str().c_str(), 204, 305, 255, 255, 255);

    datos.str("");
    datos << "(Pulse confirmar para continuar)";
    textos.mostrarTexto(v, datos.str().c_str(), 64, 370, 255, 255, 255);

    if (reservado(nombre)) {
      datos.str("");
      datos << "Nombre reservado, escoga otro";
      textos.mostrarTexto(v, datos.str().c_str(), 104, 435, 255, 255, 255);
    }
  }


  v->actualizar();
}

bool GestorPartida::areaIncremento(const Puntero& p)
{
return ((p.posX() >= INC_X && p.posX() < INC_XW) && (p.posY() >= INC_Y && p.posY() < INC_YH));
}

bool GestorPartida::areaDecremento(const Puntero& p)
{
return ((p.posX() >= DEC_X && p.posX() < DEC_XW) && (p.posY() >= DEC_Y && p.posY() < DEC_YH));
}

char GestorPartida::caracter(SDL_Event evento)
{
  if (evento.key.keysym.sym == SDLK_a)
    return 'A';
  else if (evento.key.keysym.sym == SDLK_b)
    return 'B';
  else if (evento.key.keysym.sym == SDLK_c)
    return 'C';
  else if (evento.key.keysym.sym == SDLK_d)
    return 'D';
  else if (evento.key.keysym.sym == SDLK_e)
    return 'E';
  else if (evento.key.keysym.sym == SDLK_f)
    return 'F';
  else if (evento.key.keysym.sym == SDLK_g)
    return 'G';
  else if (evento.key.keysym.sym == SDLK_h)
    return 'H';
  else if (evento.key.keysym.sym == SDLK_i)
    return 'I';
  else if (evento.key.keysym.sym == SDLK_j)
    return 'J';
  else if (evento.key.keysym.sym == SDLK_k)
    return 'K';
  else if (evento.key.keysym.sym == SDLK_l)
    return 'L';
  else if (evento.key.keysym.sym == SDLK_m)
    return 'M';
  else if (evento.key.keysym.sym == SDLK_n)
    return 'N';
  else if (evento.key.keysym.sym == SDLK_o)
    return 'O';
  else if (evento.key.keysym.sym == SDLK_p)
    return 'P';
  else if (evento.key.keysym.sym == SDLK_q)
    return 'Q';
  else if (evento.key.keysym.sym == SDLK_r)
    return 'R';
  else if (evento.key.keysym.sym == SDLK_s)
    return 'S';
  else if (evento.key.keysym.sym == SDLK_t)
    return 'T';
  else if (evento.key.keysym.sym == SDLK_u)
    return 'U';
  else if (evento.key.keysym.sym == SDLK_v)
    return 'V';
  else if (evento.key.keysym.sym == SDLK_w)
    return 'W';
  else if (evento.key.keysym.sym == SDLK_x)
    return 'X';
  else if (evento.key.keysym.sym == SDLK_y)
    return 'Y';
  else if (evento.key.keysym.sym == SDLK_z)
    return 'Z';
  else if (evento.key.keysym.sym == SDLK_0)
    return '0';
  else if (evento.key.keysym.sym == SDLK_1)
    return '1';
  else if (evento.key.keysym.sym == SDLK_2)
    return '2';
  else if (evento.key.keysym.sym == SDLK_3)
    return '3';
  else if (evento.key.keysym.sym == SDLK_4)
    return '4';
  else if (evento.key.keysym.sym == SDLK_5)
    return '5';
  else if (evento.key.keysym.sym == SDLK_6)
    return '6';
  else if (evento.key.keysym.sym == SDLK_7)
    return '7';
  else if (evento.key.keysym.sym == SDLK_8)
    return '8';
  else if (evento.key.keysym.sym == SDLK_9)
    return '9';

  else 
    return '+';
}

bool GestorPartida::reservado(string nombre)
{
  bool bandera = false;

  for (Jugadores::iterator i = jugadores.begin(); i != jugadores.end(); i++) {
    if ((*i)->nombre() == nombre || nombre == "robotuno" || nombre == "robotdos" || nombre == "robotres")
      bandera = true;
  }

  return bandera;
}

void GestorPartida::desordenarJugadores()
{
  srand(time(0)); // Iniciamos la semilla

  random_shuffle(jugadores.begin(), jugadores.end());
}

void GestorPartida::incrementarPosicion()
{
  Jugador * aux;

  aux = jugadores[0];

  for (unsigned i = 0; i < numJugadores - 1; i++)
    jugadores[i] = jugadores[i+1];

  jugadores[numJugadores - 1] = aux;
}

void GestorPartida::pantallaRonda(const unsigned r, Video* v)
{
  stringstream datos;
  fondo->dibujar(v);

  datos.str("");
  datos << "RONDA " << r;
  ronda.cursiva();
  ronda.mostrarTexto(v, datos.str().c_str(), 300, 300, 255, 255, 255);

  v->actualizar();

  sleep(3);
}

bool areaConfirmar(const Puntero& p)
{
return ((p.posX() >= CONFIRMAR_X && p.posX() < CONFIRMAR_XW) && (p.posY() >= CONFIRMAR_Y && p.posY() < CONFIRMAR_YH));
}
bool areaCancelar(const Puntero& p)
{
  return ((p.posX() >= CANCELAR_X && p.posX() < CANCELAR_XW) && (p.posY() >= CANCELAR_Y && p.posY() < CANCELAR_YH));
}
