#ifndef _IMAGEN_H_
#define _IMAGEN_H_

#include <SDL/SDL.h>
#include <SDL/SDL_image.h>
#include "video.h"


/** Clase Imagen. Clase para gestionar el trabajo con imágenes.
*
* Distribuido bajo licencia GNU GPLv.3
*
* @author Daniel Chaves Vázquez
* @version 1.0
*/
class Imagen {
 public:
 
  // Constructor

  /** Constructor de la clase Imagen
   * @param ruta Ruta del fichero de imagen que se va a cargar.
   * @param x Indica la coordenada en el eje x de la imagen.
   * @param y Indica la coordenada en el eje y de la imagen.
   */
  Imagen(const char *ruta, Sint16 x = 0, Sint16 y = 0);

  // Modificadores

  /** Método que dibuja una imagen en una superficie.
   * @param v Puntero a un objeto de la clase que contiene la superficie donde se va a dibujar la imagen.
   */	
  void dibujar(Video *v);

  /** Método que modifica la coordenada x de la imagen.
   * @param x_ Indica la nueva coordenada en el eje x de la imagen.
   */
  void modificarX(const Sint16 x_);

  /** Método que modifica la coordenada y de la imagen.
   * @param y_ Indica la nueva coordenada en el eje y de la imagen.
   */
  void modificarY(const Sint16 y_);

  // Observadores

  /** Método que devuelve la coordenada x de la imagen.
   * @return Devuelve la coordenada en el eje x de la imagen.
   */
  const Sint16 obtenerX() const;

  /** Método que devuelve la coordenada y de la imagen.
   * @return Devuelve la coordenada en el eje y de la imagen.
   */
  const Sint16 obtenerY() const;

  /** Método que devuelve el ancho de la imagen.
   * @return Devuelve el ancho de la imagen.
   */
  const Uint16 ancho() const;

  /** Método que devuelve el alto de la imagen.
   * @return Devuelve el alto de la imagen.
   */
  const Uint16 alto() const;

  /** Destructor de la clase Imagen.
   */
  ~Imagen();
 
 private:
  SDL_Surface *imagen;
  SDL_Rect posicion;
};

inline void Imagen::dibujar(Video *v) { SDL_BlitSurface(imagen, NULL, v->superficie(), &posicion); }
inline void Imagen::modificarX(const Sint16 x_) { posicion.x = x_; }
inline void Imagen::modificarY(const Sint16 y_) { posicion.y = y_; }
inline const Sint16 Imagen::obtenerX() const { return posicion.x; }
inline const Sint16 Imagen::obtenerY() const { return posicion.y; }
inline const Uint16 Imagen::ancho() const { return posicion.w; }
inline const Uint16 Imagen::alto() const { return posicion.h; }
inline Imagen::~Imagen() { SDL_FreeSurface(imagen); }

#endif
