#include <iostream>
#include <sstream>
#include <fstream>
#include "constantes.h"
#include "video.h"
#include "puntero.h"
#include "monton.h"
#include "soporte.h"
#include "jugador.h"

using namespace std;

Jugador::Jugador(const string& nombre, bool robt): robot(robt) 
{
  if (robot) { // Si es un robot, solo se crea en memoria, ya que no vamos a guardar ninguna puntuación ni tenemo que cargarla.
    nom = nombre;
    puntPartida = 0;
    puntGeneral = 0;
    partGanadas = 0;
    partPerdidas = 0;
    partAbandonadas = 0;
    porcentajeGanadas = 0.0;
    porcentajePerdidas = 0.0;
    porcentajeAbandonadas = 0.0;
    rango = NOVEL;
  }

  else {
    // Variables para el uso de ficheros
    fstream jugadores, Perfil;
    
    // Variables auxiliares
    char buffer[10];
    string cBuffer, ruta, datos, nombres;
    bool encontrado = false;
    
    // Abrimos el archivo para lectura
    jugadores.open("./guardado/gestorJugadores.sav", fstream::in);
    
    nombres = "";
    
    // Mientras no hayamos llegado al final del fichero y no hayamos encontrado el nombre que buscamos
    while (!jugadores.eof() && !encontrado) {
      jugadores.getline(buffer, 10, '\n');
      cBuffer = buffer;
      if (cBuffer == nombre)
	encontrado = true;
      else if (cBuffer != "") // Para no meter lineas vacias
	nombres += cBuffer + "\n";
    }      
    jugadores.close();
    
    // Si no se ha encontrado un jugador con ese nombre, lo creamos
    if (!encontrado) {
      // Añadimos a la lista de nombres el nombre del nuevo jugador.
      nombres += nombre + "\n";
      
      jugadores.open("./guardado/gestorJugadores.sav", fstream::out);
      jugadores.write(nombres.c_str(), nombres.size());    
      jugadores.close();
      
      ruta = "./guardado/" + nombre + ".sav";
      Perfil.open(ruta.c_str(), fstream::out);
      
      //Los datos son: nombre / punt General / part Ganadas / part Perdidas / part Abandonadas
      datos = nombre + "\n0\n0\n0\n0\n";
      Perfil.write(datos.c_str(), datos.size());
      Perfil.close();
      nom = nombre;
      puntPartida = 0;
      puntGeneral = 0;
      partGanadas = 0;
      partPerdidas = 0;
      partAbandonadas = 0;
      porcentajeGanadas = 0.0;
      porcentajePerdidas = 0.0;
      porcentajeAbandonadas = 0.0;
      rango = NOVEL;
    }
    
    // Si se ha encontrado
    else if (encontrado) {
      ruta = "./guardado/" + nombre + ".sav";
      Perfil.open(ruta.c_str(), fstream::in);
      
      //Los datos son: nombre / punt General / part Ganadas / part Perdidas / part Abandonadas
      Perfil.getline(buffer, '\n');
      nom = buffer;
      Perfil.getline(buffer, '\n');
      puntGeneral = atoi(buffer);
      Perfil.getline(buffer, '\n');
      partGanadas = atoi(buffer);
      Perfil.getline(buffer, '\n');
      partPerdidas = atoi(buffer);
      Perfil.getline(buffer, '\n');
      partAbandonadas = atoi(buffer);
      Perfil.close();
      
      // Datos derivados
      double total = partGanadas + partPerdidas + partAbandonadas;
      if (total != 0) {
	porcentajeGanadas = partGanadas * 100 / total;
	porcentajePerdidas = partPerdidas * 100 / total;
	porcentajeAbandonadas = partAbandonadas * 100 / total;
      }
      else {
	porcentajeGanadas = 0;
	porcentajePerdidas = 0;
	porcentajeAbandonadas = 0;
      }
      
      if (puntGeneral <= 250) rango = NOVEL;
      else if (puntGeneral > 250 && puntGeneral <= 500) rango = MEDIO;
      else if (puntGeneral > 500 && puntGeneral <= 1000) rango = AVANZADO;
      else if (puntGeneral > 1000) rango = EXPERTO;
    }
    
    //else 
    //cout << "ERROR en constructor de JUGADOR";
  }
}

void Jugador::guardarJugador()
{
  if (!robot) {
    // Variables para el uso de ficheros
    fstream jugadores;
    fstream Perfil;
    
    // Variables auxiliares
    char buffer[10];
    string cBuffer, ruta, datos;
    bool encontrado = false;
    stringstream ss;
    
    jugadores.open("./guardado/gestorJugadores.sav", fstream::in);
    
    while (!jugadores.eof() && !encontrado) {
      jugadores.getline(buffer, 10, '\n');
      cBuffer = buffer;
      if (cBuffer == nom)
	encontrado = true;
    }
    
    jugadores.close();
    
    if (encontrado) {
      ss << puntGeneral << "\n" 
	 << partGanadas << "\n" 
	 << partPerdidas << "\n" 
	 << partAbandonadas << "\n";
      
      datos = nom + '\n' + ss.str();
      
      ruta = "./guardado/" + nom + ".sav";
      
      Perfil.open(ruta.c_str(), fstream::out);  
      Perfil.write(datos.c_str(), datos.size());
      Perfil.close();
    }
    //else
    //cout << "Error: " << nom << " no existe en el sistema" << endl;
  }

  //else
  //cout << "Era un robot, sin cambios" << endl;
}

void Jugador::borrarJugador()
{
  // Variables para el uso de ficheros
  fstream jugadores;

  // Variables auxiliares
  char buffer[10];
  string cBuffer;
  string orden, nombres;
  bool encontrado = false;

  jugadores.open("./guardado/gestorJugadores.sav", fstream::in);

  nombres = "";
  
  while (!jugadores.eof()) {
    jugadores.getline(buffer, 10, '\n');
    cBuffer = buffer;
    if (cBuffer == nom)
      encontrado = true;
    else if (cBuffer != "") // Para no meter lineas vacias
      nombres += cBuffer + "\n";
  }

  jugadores.close();
  
  if (encontrado) {
    orden = "rm ./guardado/" +  nom + ".sav";
    system(orden.c_str());
    
    jugadores.open("./guardado/gestorJugadores.sav", fstream::out);
    if (nombres.size() > 0)
      jugadores.write(nombres.c_str(), nombres.size());
    else
      jugadores.write("", 0);

    jugadores.close();
  }
  //else
  //cout << "Error: " << nom << " no existe en el sistema" << endl;
}

void Jugador::partidaGanada()
{
  partGanadas++;

  int totalPartidas = partGanadas+partPerdidas+partAbandonadas;

  porcentajeGanadas = partGanadas * 100 / totalPartidas;
  porcentajePerdidas = partPerdidas * 100 / totalPartidas;
  porcentajeAbandonadas = partAbandonadas * 100 / totalPartidas;
}

void Jugador::partidaPerdida() 
{
  partPerdidas++;

  int totalPartidas = partGanadas+partPerdidas+partAbandonadas;

  porcentajeGanadas = partGanadas * 100 / totalPartidas;
  porcentajePerdidas = partPerdidas * 100 / totalPartidas;
  porcentajeAbandonadas = partAbandonadas * 100 / totalPartidas;
}

void Jugador::partidaAbandonada() 
{ 
  partAbandonadas++;

  int totalPartidas = partGanadas+partPerdidas+partAbandonadas;

  porcentajeGanadas = partGanadas * 100 / totalPartidas;
  porcentajePerdidas = partPerdidas * 100 / totalPartidas;
  porcentajeAbandonadas = partAbandonadas * 100 / totalPartidas;
 }

void Jugador::puntuacionPartida(int puntos, int situacion)
{
  if (situacion == 1)
    puntPartida = puntos;
  else
    puntPartida = -puntos;

  puntGeneral += puntPartida;

  if (puntGeneral <= 250) rango = NOVEL;
  else if (puntGeneral > 250 && puntGeneral <= 500) rango = MEDIO;
  else if (puntGeneral > 500 && puntGeneral <= 1000) rango = AVANZADO;
  else if (puntGeneral > 1000) rango = EXPERTO;
}

void Jugador::iniciarSoporte(Monton& m)
{
  int p = 0;

  while(!soporte.vacio() || p < CAPACIDAD) {
    soporte.escogerFicha(p);
    p++;
  }

  for (int i = 0; i < 14; i++)
    soporte.robar(m);
}


