#ifndef _MENU_H_
#define _MENU_H_

#include "video.h"
#include "imagen.h"
#include "fuente.h"
#include "puntero.h"
#include "sonido.h"

/** Clase Menu. Clase para gestionar el menú principal y los submenús.
*
* Distribuido bajo licencia GNU GPLv.3
*
* @author Daniel Chaves Vázquez
* @version 1.0
*/
class Menu
{
 public:

  // Constructor

  /** Constructor de la clase Menu.
   */
  Menu();

  // Modificadores

  /** Método para gestionar el menú principal.
   * @param v Puntero a un objeto de la clase que contiene la superficie donde se van a dibujar las imagenes.
   * @return Devuelve la opción escogida (Jugar, Jugadores o Salir).
   */
  int menuPpal(Video *v);

  /** Método para gestionar el submenú Jugar.
   * @param v Puntero a un objeto de la clase que contiene la superficie donde se van a dibujar las imagenes.
   * @return Devuelve la opción escogida (Simple, Concurso o Volver).
   */
  int menuJugar(Video *v);

  /** Método para gestionar el submenú Jugadores.
   * @param v Puntero a un objeto de la clase que contiene la superficie donde se van a dibujar las imagenes.
   * @return Devuelve la opción escogida (Ver, Borrar o Volver).
   */
  int menuJugadores(Video *v);

  /** Destructor de la clase Menu.
   */
  ~Menu();

 private:
  Imagen *fondo, *jugarN, *jugarR, *jugadoresN, *jugadoresR, *salirN, *salirR, *concursoN, *concursoR, *simpleN, *simpleR, *volverN, *volverR, *verN, *verR, *borrarN, *borrarR, *suena, *nosuena;

  Efecto opcion;
  Fuente descripcion;

  /** Método que muestra la pantalla para ver el menú principal.
   * @param v Puntero a un objeto de la clase que contiene la superficie donde se van a dibujar las imagenes. 
   */
  void actualizarPpal(Video *v);

  /** Método que muestra la pantalla para ver el submenú Jugar.
   * @param v Puntero a un objeto de la clase que contiene la superficie donde se van a dibujar las imagenes. 
   */
  void actualizarJugar(Video *v);

  /** Método que muestra la pantalla para ver el submenú Jugadores.
   * @param v Puntero a un objeto de la clase que contiene la superficie donde se van a dibujar las imagenes. 
   */
  void actualizarJugadores(Video *v);

  /** Método que devuelve si las coordenadas de un puntero están dentro del area del botón Jugar o no.
   * @param p Referencia constante a un objeto de la clase Puntero.
   * @return Devuelve si las coordenadas del puntero están o no en el area del botón Jugar.
   */
  bool areaJugar(const Puntero& p);

  /** Método que devuelve si las coordenadas de un puntero están dentro del area del botón Jugadores o no.
   * @param p Referencia constante a un objeto de la clase Puntero.
   * @return Devuelve si las coordenadas del puntero están o no en el area del botón Jugadores.
   */
  bool areaJugadores(const Puntero& p);

  /** Método que devuelve si las coordenadas de un puntero están dentro del area del botón Salir o no.
   * @param p Referencia constante a un objeto de la clase Puntero.
   * @return Devuelve si las coordenadas del puntero están o no en el area del botón Salir.
   */
  bool areaSalir(const Puntero& p);

  /** Método que devuelve si las coordenadas de un puntero están dentro del area del botón Simple o no.
   * @param p Referencia constante a un objeto de la clase Puntero.
   * @return Devuelve si las coordenadas del puntero están o no en el area del botón Simple.
   */
  bool areaSimple(const Puntero& p);

  /** Método que devuelve si las coordenadas de un puntero están dentro del area del botón Concurso o no.
   * @param p Referencia constante a un objeto de la clase Puntero.
   * @return Devuelve si las coordenadas del puntero están o no en el area del botón Concurso.
   */
  bool areaConcurso(const Puntero& p);

  /** Método que devuelve si las coordenadas de un puntero están dentro del area del botón Volver o no.
   * @param p Referencia constante a un objeto de la clase Puntero.
   * @return Devuelve si las coordenadas del puntero están o no en el area del botón Volver.
   */
  bool areaVolver(const Puntero& p);

  /** Método que devuelve si las coordenadas de un puntero están dentro del area del botón Ver o no.
   * @param p Referencia constante a un objeto de la clase Puntero.
   * @return Devuelve si las coordenadas del puntero están o no en el area del botón Ver.
   */
  bool areaVer(const Puntero& p);

  /** Método que devuelve si las coordenadas de un puntero están dentro del area del botón Borrar o no.
   * @param p Referencia constante a un objeto de la clase Puntero.
   * @return Devuelve si las coordenadas del puntero están o no en el area del botón Borrar.
   */
  bool areaBorrar(const Puntero& p);
};

#endif
