#ifndef _MONTON_H_
#define _MONTON_H_

#include <vector>
#include "puntero.h"
#include "ficha.h"

using namespace std;

/** Clase Monton. Clase para el manejo y funcionamiento del montón del juego.
*
* Distribuido bajo licencia GNU GPLv.3
*
* @author Daniel Chaves Vázquez
* @version 1.0
*/
class Monton {
 public:

  // Constructor

  /** Constructor de la clase Monton. Se encarga de crear todas las fichas del juego y desordenarlas.
   */
  Monton();

  // Modificadores

  /** Método encargada de sacar una ficha del montón.
   * @return Devuelve una referencia constante a un objeto de la clase Ficha.
   */
  const Ficha& extraer();

  // Observadores
  
  /** Método que devuelve si el montón está vacio o no.
   * @return Devuelve si el montón está vacio o no.
   */
  bool vacio() const;

  /** Destructor de la clase Monton.
   */
  ~Monton();

 private:
  int fichRest;
  Ficha sigExt;
  typedef vector<Ficha> monton;
  monton mon;

  // Métodos privados.

  /** Método que desordena el monton de fichas de manera aleatoria.
   */
  void desordenar();
};

// Funciones externas

/** Función que devuelve si las coordenadas de un puntero están dentro del area del montón o no.
 * @param p Referencia constante a un objeto de la clase Puntero.
 * @return Devuelve si las coordenadas del puntero están o no en el area del montón.
 */
bool areaMonton(const Puntero& p);

inline bool Monton::vacio() const { return mon.empty(); }
inline Monton::~Monton() {}

#endif
