#include <iostream>
#include <sstream>
#include <SDL/SDL.h>
#include "constantes.h"
#include "puntero.h"
#include "video.h"
#include "imagen.h"
#include "ficha.h"
#include "monton.h"
#include "jugador.h"
#include "tablero.h"
#include "cronometro.h"
#include "partida.h"
#include "sonido.h"
#include "IA.h"
#include "gestorPartida.h"

extern bool silencio;

using namespace std;

Partida::Partida(unsigned numJug): numJugadores(numJug), jugadorActual(0), f(false, 0), robar(false), comprobar(false), salir(false), turn(1), cont(0), pausa(false), cron("./fuentes/DS-DIGI.TTF", 60), info("./fuentes/LearningCurve.ttf", 30), cambio("./fuentes/IMPACT.TTF", 50), correcto("./sonidos/correcto.wav"), incorrecto("./sonidos/incorrecto.wav"), reloj("./sonidos/tictac.wav"), montonfichas("./sonidos/monton.wav"), coger("./sonidos/cogerficha.wav"), soltar("./sonidos/soltarficha.wav"), musica("sonidos/partida.wav") {
  fondo = new Imagen("./imagenes/partida/fondo.png");
  fMenu = new Imagen("./imagenes/menus/fondoMenu.png");
  iGan = new Imagen("./imagenes/partida/ganar.png", 96, 138);
  iAba = new Imagen("./imagenes/partida/abandonar.png", 96, 138);

  // Altavoz cancelacion de sonido
  suena = new Imagen("./imagenes/sonando.png", 0, 0);
  nosuena = new Imagen("./imagenes/silencio.png", 0 , 0); 

 // Confirmar-Cancelar
  confirmar = new Imagen("./imagenes/confirmar.png", CONFIRMAR_X, CONFIRMAR_Y);
  cancelar = new Imagen("./imagenes/cancelar.png", CANCELAR_X, CANCELAR_Y);
}

void Partida::jugar(Video *v, int tipoPartida)
{
  // Variables auxiliares
  int fin = 0, puntos = 0;
  unsigned jugadorGanador;
  IA r;

  // Comienza la música
  musica.reproducir();

  // INICIALIZACION DE LAS VARIABLES PARA LA PARTIDA
  // Inicializamos jugadores
  iniciarJugadores();

  pantallaCambio(v);
  // INICIO DEL BUCLE DEL JUEGO
  do {
    if (!jugadores[jugadorActual]->jRobot())
      fin = turno(v);
    // Parte IA
    else {
      tableroCopia = tablero;
      soporteCopia = jugadores[jugadorActual]->obtenerSoporte();

      int fichas = soporteCopia.fichas();
      int suma = soporteCopia.sumaSoporte();

      fin = r.selectorIA(&soporteCopia, &tableroCopia, numJugadores - 1);

      // si eltablero está bien
      if (tableroCopia.comprobar()) { //cout << " se pulso COMPROBAR y el tablero está bien ";
	// Si el jugador está abierto
	if (soporteCopia.abierto()) { //cout << "y el jugador esta abierto ";
	  // Si ha puesto alguna ficha
	  if (soporteCopia.fichas() < fichas) { //cout << "y ha puesto alguna ficha: OK" << endl;
	    // Entonces todo ha ido bien y si pueden actualizar los atributos originales.
	    jugadores[jugadorActual]->obtenerSoporte() = soporteCopia;
	    tablero = tableroCopia;
	    correcto.reproducir();
	  }
	  // Si no es que no se ha puesto nada y hay que robar
	  else { //cout << " y no ha puesto nada: Roba 1" << endl;
	    jugadores[jugadorActual]->obtenerSoporte().robar(monton);
	    incorrecto.reproducir();
	  }
	}
	// Si no está abierto
	else { //cout << "y el jugador NO esta abierto ";
	  int diferencia = suma - soporteCopia.sumaSoporte();
	  
	  // Si ha puesto más de 30 puntos
	  if (diferencia >= 30) { //cout << "y ha puesto 30 puntos o más: OK" << endl;
	    // Entonces todo ha ido bien y si pueden actualizar los atributos originales.
	    jugadores[jugadorActual]->obtenerSoporte() = soporteCopia;
	    tablero = tableroCopia;
	    jugadores[jugadorActual]->obtenerSoporte().abrir();
	    correcto.reproducir();
	  }
	// Si no, roba
	  else { //cout << "y no ha puesto 30 puntos: Roba 1" << endl;
	    jugadores[jugadorActual]->obtenerSoporte().robar(monton);
	    incorrecto.reproducir();
	  }
	}
      }
      else if (!tableroCopia.comprobar()) { //cout << " se pulso COMPROBAR y el tablero NO está bien ";
	jugadores[jugadorActual]->obtenerSoporte().robar(monton);
	jugadores[jugadorActual]->obtenerSoporte().robar(monton);
	jugadores[jugadorActual]->obtenerSoporte().robar(monton);
	incorrecto.reproducir();
      }

      sleep (2);
    }
      
    if (fin == 0) {
      incrementarJugadorActual();
      // Entramos en pantalla de cambio
      pantallaCambio(v);
      if (jugadorActual == 0)
	turn++;
    }
  } while (fin == 0);

  // Animación final
  if (fin == 1)
    pantallaGanador(v);
  else
    pantallaAbandonado(v);

  
  if (tipoPartida > 0) {
    jugadorGanador = jugadorActual;

    // Si el jugador actual ha ganado
    if (fin == 1) {
      jugadores[jugadorActual]->partidaGanada();
      
      incrementarJugadorActual();
      
      // Actualizamos las puntuaciones de los demás jugadores y obtenemos los puntos del ganador
      while (jugadorActual != jugadorGanador) {
	jugadores[jugadorActual]->partidaPerdida();
	jugadores[jugadorActual]->puntuacionPartida(jugadores[jugadorActual]->obtenerSoporte().sumaSoporte(), 0);
	
	jugadores[jugadorActual]->guardarJugador();
	
	puntos += jugadores[jugadorActual]->obtenerSoporte().sumaSoporte();
	incrementarJugadorActual();
      }
      
      jugadores[jugadorActual]->puntuacionPartida(puntos, 1);
      
      jugadores[jugadorActual]->guardarJugador();
    }
    
    // Si abandona la partida
    else if (fin == -1) {
      jugadores[jugadorActual]->partidaAbandonada();
      puntos += jugadores[jugadorActual]->obtenerSoporte().sumaSoporte();
      
      incrementarJugadorActual();
      
      // Actualizamos las puntuaciones de los demás jugadores y obtenemos los puntos del ganador
      while (jugadorActual != jugadorGanador) {
	jugadores[jugadorActual]->partidaGanada();
	jugadores[jugadorActual]->puntuacionPartida(jugadores[jugadorActual]->obtenerSoporte().sumaSoporte(), 1);
	
	jugadores[jugadorActual]->guardarJugador();
	
	puntos += jugadores[jugadorActual]->obtenerSoporte().sumaSoporte();
	incrementarJugadorActual();
      }
      
      jugadores[jugadorActual]->puntuacionPartida(puntos, -1);
      
      jugadores[jugadorActual]->guardarJugador();
    }
  }
}

int Partida::turno(Video *v)
{
  // Variables auxiliares
  bool pasarTurno = false;
  int fichas, suma;
 
  pausa = false;

  cont = 0;
  robar = false;
  comprobar = false;
  salir = false;

  f = Ficha(false, 0);
  crono.iniciar();

  // Copiamos el contenido válido por si ocurren errores
  tableroCopia = tablero;
  soporteCopia = jugadores[jugadorActual]->obtenerSoporte();
  fichas = soporteCopia.fichas();
  suma = soporteCopia.sumaSoporte();
  actualizarPantalla(v);
  while (!pasarTurno && !crono.tiempoAgotado()) {
    while (SDL_PollEvent(&evento)) {
      // EVENTO DE SALIDA
      if (evento.type == SDL_QUIT)
	return -1;
      // EVENTOS DE TECLADO
      if (evento.type == SDL_KEYDOWN){
	if (evento.key.keysym.sym == SDLK_ESCAPE) { // Salir
	  crono.pausa();
	  musica.pausa();

	  salir = salirPartida(v);

	  if (salir)
	    pasarTurno = true;
	  else {
	    pasarTurno = false;
	    musica.continuar();
	    crono.continuar();
	  }
	}

	else {
	  pasarTurno = eventosTeclado();
	  actualizarPantalla(v);
	}
      }
      // Si se ha pulsado un boton del ratón
      if (evento.type == SDL_MOUSEBUTTONDOWN && !salir) {
	pPulsar.actualizar(); // guardamos la posición donde sucedió
	pasarTurno = eventosPulsacion(); // Llamamos al manejador de eventos de pulsación
	actualizarPantalla(v);	
      }
      
      if (!f.fichaNula() && evento.type == SDL_MOUSEMOTION && !pausa && !salir) {
	pMovimiento.actualizar();
	f.posicionar(pMovimiento.posX(), pMovimiento.posY());
	
	// Con esta linea vaciamos la cola de eventos para no procesar todos los movimientos y en el caso de que se haya soltado el boton del ratón a la vez que se movia, procesar correctamente donde sucedió.
	while(SDL_PollEvent(&evento)){
	  if (evento.type == SDL_MOUSEBUTTONUP) {
	    pSoltar.actualizar(); // guardamos la posición donde sucedió
	    pasarTurno = eventosLiberacion(); // Llamamos al manejador de eventos de liberación
	    actualizarPantalla(v);
	  }
	}
	actualizarPantalla(v);
      }
      
      // Si se ha soltado un botón del ratón
      if (evento.type == SDL_MOUSEBUTTONUP && !pausa && !salir) {
	pSoltar.actualizar(); // guardamos la posición donde sucedió
	pasarTurno = eventosLiberacion(); // Llamamos al manejador de eventos de liberación
	actualizarPantalla(v);
      }
    }
    if (crono.segundosTranscurridos() > cont && !pausa && !salir) {
      cont++;
      actualizarPantalla(v);
      reloj.reproducir();
    }
    // Dormimos el proceso
    SDL_Delay(1);
  }
  
  // Comprobaciones:

  // Si no se agotó el tiempo:
  if (!crono.tiempoAgotado()) { //cout << "El tiempo NO se agotó ";
    // Si pulsó salir
    if (salir)
      return -1;
    // Si pulsó robar
    // Si el jugador pulsa robar y el tablero está bien
    if (robar && tableroCopia.comprobar()) { //cout << "y se pulso ROBAR y el tablero esta bien" << endl;
      jugadores[jugadorActual]->obtenerSoporte().robar(monton);
      montonfichas.reproducir();
    }
    
    // Si el jugador pulsa robar y el tablero NO está bien
    else if (robar && !tableroCopia.comprobar()) { //cout << "y se pulso ROBAR y el tablero NO esta bien" << endl;
      jugadores[jugadorActual]->obtenerSoporte().robar(monton);
      jugadores[jugadorActual]->obtenerSoporte().robar(monton);
      jugadores[jugadorActual]->obtenerSoporte().robar(monton);
      montonfichas.reproducir();
    }
    
    // Si pulsa comprobar
    // si el jugador pulsa comprobar y el tablero está bien
    else if (comprobar && tableroCopia.comprobar()) { //cout << " se pulso COMPROBAR y el tablero está bien ";
      // Si el jugador está abierto
      if (soporteCopia.abierto()) { //cout << "y el jugador esta abierto ";
	// Si ha puesto alguna ficha
	if (soporteCopia.fichas() < fichas) { //cout << "y ha puesto alguna ficha: OK" << endl;
	  // Entonces todo ha ido bien y si pueden actualizar los atributos originales.
	  jugadores[jugadorActual]->obtenerSoporte() = soporteCopia;
	  tablero = tableroCopia;
	  correcto.reproducir();
	}
	// Si no es que no se ha puesto nada y hay que robar
	else { //cout << " y no ha puesto nada: Roba 1" << endl;
	  jugadores[jugadorActual]->obtenerSoporte().robar(monton);
	  incorrecto.reproducir();
	    }
      }
      // Si no está abierto
      else { //cout << "y el jugador NO esta abierto ";
	int diferencia = suma - soporteCopia.sumaSoporte();

	// Si ha puesto más de 30 puntos
	if (diferencia >= 30) { //cout << "y ha puesto 30 puntos o más: OK" << endl;
	  // Entonces todo ha ido bien y si pueden actualizar los atributos originales.
	  jugadores[jugadorActual]->obtenerSoporte() = soporteCopia;
	  tablero = tableroCopia;
	  jugadores[jugadorActual]->obtenerSoporte().abrir();
	  correcto.reproducir();
	}
	// Si no, roba
	else { //cout << "y no ha puesto 30 puntos: Roba 1" << endl;
	  jugadores[jugadorActual]->obtenerSoporte().robar(monton);
	  incorrecto.reproducir();
	}
      }
    }
    else if (comprobar && !tableroCopia.comprobar()) { //cout << " se pulso COMPROBAR y el tablero NO está bien ";
      jugadores[jugadorActual]->obtenerSoporte().robar(monton);
      jugadores[jugadorActual]->obtenerSoporte().robar(monton);
      jugadores[jugadorActual]->obtenerSoporte().robar(monton);
      incorrecto.reproducir();
    }
  }

  // Si el tiempo se ha agotado:
  else { //cout << "El tiempo se agoto ";
    // Si el tablero está bien
    if (tableroCopia.comprobar()) { //cout << "y el tablero está correcto: Roba 1" << endl;
	jugadores[jugadorActual]->obtenerSoporte().robar(monton); // Solo se roba una
      }

    // Si el tablero NO está bien
    else if (!tableroCopia.comprobar()) { //cout << " y el tablero no está correcto: Roba 3" << endl; 
      jugadores[jugadorActual]->obtenerSoporte().robar(monton); // Se roban tres
      jugadores[jugadorActual]->obtenerSoporte().robar(monton);
      jugadores[jugadorActual]->obtenerSoporte().robar(monton);
    }
      incorrecto.reproducir();
  }

  // Si el jugador se ha quedado sin fichas
  if (jugadores[jugadorActual]->obtenerSoporte().fichas() == 0)
    return 1;

  else
    return 0;
}

bool Partida::anadirJugador(Jugador* j)
{
  bool bandera = false; 

  if (jugadores.size() < numJugadores) {
    bandera = true;
    Jugadores::iterator i = jugadores.begin();
    while(i != jugadores.end() && bandera) {
      if ((*i)->nombre() == j->nombre()) // si ya está insertado
	bandera = false;
      else
	i++;
    }

    if (bandera)
      jugadores.push_back(j);
  }
  return bandera; // Devuelve true si inserta y false en caso contrario.
}

bool Partida::eliminarJugador(Jugador* j)
{
  bool bandera = false; 

  if (!jugadores.empty()) {
    Jugadores::iterator i = jugadores.begin();
    while(i != jugadores.end() && !bandera) {
      if ((*i)->nombre() == j->nombre()) // si se encuentra
	bandera = true;
      else
	i++;
    }

    if (bandera)
      jugadores.erase(i);
  }
  
  return bandera; // Devuelve true si elimina y false en caso contrario.
}

void Partida::iniciarJugadores()
{
  if (!jugadores.empty()) {
    for (Jugadores::iterator i = jugadores.begin(); i != jugadores.end(); i++)
      (*i)->iniciarSoporte(monton);
  }
  else
    cout << "No hay jugadores" << endl;
}

Partida::~Partida() {}

 void Partida::incrementarJugadorActual()
 {
   if (jugadorActual == numJugadores - 1)
     jugadorActual = 0;

   else
     jugadorActual++;
 }

void Partida::mostrarTextos(Video *v)
{
  string aux;
  stringstream datos;

  // INFO
  aux = "Jugador: " + jugadores[jugadorActual]->nombre();
  info.subrayado();
  info.mostrarTexto(v, aux.c_str(), 800, 580);  
  info.normal();

  datos << "Fichas Restantes: " << soporteCopia.fichas();
  info.mostrarTexto(v, datos.str().c_str(), 800, 610);

  datos.str(""); 

  datos << "Puntos: " << soporteCopia.sumaSoporte(); 
  info.mostrarTexto(v, datos.str().c_str(), 800, 640);

  datos.str(""); 

  datos << "Turno: " << turn; 
  info.mostrarTexto(v, datos.str().c_str(), 800, 670);

  // CRONOMETRO

  if (pausa) {
    datos.str("");
    datos << "P";
    cron.mostrarTexto(v, datos.str().c_str(), 967, 38);
  }

  else {
    datos.str("");
    datos << cont;
    // Posicionamiento según los segundos transcurridos
    if (cont < 10)
      cron.mostrarTexto(v, datos.str().c_str(), 967, 38);
    else if(cont >= 10 && cont < 20)
      cron.mostrarTexto(v, datos.str().c_str(), 957, 38);
    else if(cont >= 20 && cont < 50)
      cron.mostrarTexto(v, datos.str().c_str(), 953, 38);
    else if (cont >= 50)
      cron.mostrarTexto(v, datos.str().c_str(), 953, 38, 255, 0, 0);
  }
}

void Partida::actualizarPantalla(Video *v)
{
  fondo->dibujar(v);

  if (!silencio)
    suena->dibujar(v);
  else
    nosuena->dibujar(v);

  mostrarTextos(v);
  tableroCopia.mostrarTablero(v);
  soporteCopia.mostrarSoporte(v);
  if (!f.fichaNula())
    f.mostrarFicha(v);
  v->actualizar();
}

void Partida::pantallaCambio(Video *v)
{
  stringstream datos;
  
  fondo->dibujar(v);

  if (!silencio)
    suena->dibujar(v);
  else
    nosuena->dibujar(v);

  tablero.mostrarTablero(v);
  datos.str("");
  datos << "Siguiente jugador: ";
  cambio.cursiva();
  cambio.mostrarTexto(v, datos.str().c_str(), 320, 650, 255, 255, 255);
  v->actualizar();
  sleep(2);

  fondo->dibujar(v);

  if (!silencio)
    suena->dibujar(v);
  else
    nosuena->dibujar(v);

  tablero.mostrarTablero(v);
  datos.str("");
  datos  << jugadores[jugadorActual]->nombre();
  cambio.mostrarTexto(v, datos.str().c_str(), 400, 650, 255, 255, 255);
  v->actualizar();
  sleep(3);
}

void Partida::pantallaGanador(Video *v)
{
 stringstream datos;
  
  fMenu->dibujar(v);
  datos.str("");
  datos << jugadores[jugadorActual]->nombre();
  cambio.cursiva();
  cambio.mostrarTexto(v, datos.str().c_str(), 350, 250, 255, 255, 255);

  datos.str("");
  datos << "HA GANADO!";
  cambio.cursiva();
  cambio.mostrarTexto(v, datos.str().c_str(), 350, 350, 255, 255, 255);

  iGan->dibujar(v);

  v->actualizar();
  sleep(3);
}

void Partida::pantallaAbandonado(Video *v)
{
 stringstream datos;
  
  fMenu->dibujar(v);
  datos.str("");
  datos << jugadores[jugadorActual]->nombre();
  cambio.cursiva();
  cambio.mostrarTexto(v, datos.str().c_str(), 350, 250, 255, 255, 255);

  datos.str("");
  datos << "HA ABANDONADO!";
  cambio.cursiva();
  cambio.mostrarTexto(v, datos.str().c_str(), 350, 350, 255, 255, 255);

  iAba->dibujar(v);

  v->actualizar();
  sleep(3);
}

bool Partida::salirPartida(Video *v)
{
  SDL_Event ev;
  Puntero p;
  int opc = 0;
  bool bandera = false;
  
  // Bucle de seleccion del número de jugadores
  while (opc == 0) {
    pantallaSalir(v);
    while (SDL_PollEvent(&ev)) {    
      if (ev.type == SDL_QUIT)
	opc = 1; // Salir  
            
      else if (ev.type == SDL_MOUSEBUTTONDOWN) {
	p.actualizar();
	
	if (p.boton() == 1 && areaConfirmar(p)) {
	  opc = 1;
	  bandera = true;
	}
	
	else if (p.boton() == 1 && areaCancelar(p)) {
	  opc = 1;
	}
      }
      SDL_Delay(1);
    }
  }
  
  return bandera;
}
 
void Partida::pantallaSalir(Video *v)
{
  stringstream datos;
  
  fMenu->dibujar(v);

  confirmar->dibujar(v);
  cancelar->dibujar(v);

  datos.str("");
  datos << "Esta seguro de que desea";
  cambio.cursiva();
  cambio.mostrarTexto(v, datos.str().c_str(), 100, 200, 255, 255, 255);

  datos.str("");
  datos << "abandonar la partida?";
  cambio.cursiva();
  cambio.mostrarTexto(v, datos.str().c_str(), 150, 300, 255, 255, 255);

  v->actualizar();
}

bool Partida::eventosPulsacion()
 {
   bool finTurno = false;

   // Si se pulsó el altavoz, alternamos entre música y silencio
   if (areaAltavoz(pPulsar) && pPulsar.boton() == 1  && !pausa) {
     musica.silenciar();
   }

   // Si se pulsó en el cronometro, ponemos en pausa
   else if (areaCronometro(pPulsar) && pPulsar.boton() == 1) {
     pausa = !pausa;
     if (pausa) {
       crono.pausa();
       musica.pausa();
     }
     else {
       crono.continuar();
      musica.continuar();
     }
   }

   // Si se pulsó en el boton "ordenar color", ordenamos por color
   else if (areaColor(pPulsar) && pPulsar.boton() == 1  && !pausa) {
     soporteCopia.ordenarColor();
     jugadores[jugadorActual]->obtenerSoporte() = soporteCopia;
   }

   // Si se pulsó en el boton "ordenar numero", ordenamos por color
   else if (areaNumero(pPulsar) && pPulsar.boton() == 1  && !pausa) {
     soporteCopia.ordenarNumero();
     jugadores[jugadorActual]->obtenerSoporte() = soporteCopia;
   } 

   // Si se pulsó en el boton "comprobar", comprobaremos
   else if (areaComprobar(pPulsar) && pPulsar.boton() == 1  && !pausa) {
     finTurno = true;
     comprobar = true;
   }

   // Si se pulsó en el montón, robaremos.
   else if (areaMonton(pPulsar) && pPulsar.boton() == 1  && !pausa) {
     finTurno = true;
     robar = true;
   }
   
   // Si se pulsó en el soporte escogemos la ficha del soporte
   else if (areaSoporte(pPulsar) && pPulsar.boton() == 1  && !pausa) {
     f = Ficha(false, 0);
     f = soporteCopia.escogerFicha(posSoporte(pPulsar));
     if (!f.fichaNula())
       coger.reproducir();
   }
   
   // Si se pulsó en el tablero escogemos la ficha del tablero
   else if (areaTablero(pPulsar) && pPulsar.boton() == 1  && !pausa) {
     f = Ficha(false, 0);
     f = tableroCopia.escogerFicha(posXTablero(pPulsar), posYTablero(pPulsar));
     if (!f.fichaNula())
       coger.reproducir();
   } 
   return finTurno;
 }

bool Partida::eventosLiberacion()
{
  // Si tenemos una ficha escogida
  if (!f.fichaNula()) {

    // Si se soltó en el area del soporte
    if (areaSoporte(pSoltar)) {
      // Si la ficha venia del soporte 
      if (areaSoporte(pPulsar)) {
	// Si se colocó en un sitio válido
	if (soporteCopia.ordenar(f, posSoporte(pSoltar)))
	  jugadores[jugadorActual]->obtenerSoporte() = soporteCopia;

	// si no se pudo colocar en la nueva posicion dejamos la ficha en la posición de la que provenia.
	else
	  soporteCopia.ordenar(f, posSoporte(pPulsar));
      }
      // Si la ficha venia del tablero, se está cometiendo una infracción, no se dejará
      else if (areaTablero(pPulsar))
	// dejamos la ficha en la posición de la que provenia.
	tableroCopia.insertarFicha(f, posXTablero(pPulsar), posYTablero(pPulsar));
    }
    
    // Si se soltó en el area del tablero
    else if (areaTablero(pSoltar)) {
      // Si la ficha venia del tablero y no se pudo colocar en la nueva posicion
      if (areaTablero(pPulsar) && !tableroCopia.insertarFicha(f, posXTablero(pSoltar), posYTablero(pSoltar)))
	// dejamos la ficha en la posición de la que provenia.
	tableroCopia.insertarFicha(f, posXTablero(pPulsar), posYTablero(pPulsar));
      
      // Si la ficha venia del soporte y no se pudo colocar en la nueva posicion
      else if (areaSoporte(pPulsar) && !tableroCopia.insertarFicha(f, posXTablero(pSoltar), posYTablero(pSoltar)))
	// dejamos la ficha en la posición de la que provenia.
	soporteCopia.ordenar(f, posSoporte(pPulsar));
    }
    
    else { // Si no la a soltado en un sitio valido:
      
      if (areaSoporte(pPulsar)) // Si venia del soporte
	soporteCopia.ordenar(f, posSoporte(pPulsar)); // se deja donde estaba.
      
      else if((areaTablero(pPulsar))) // Si venia del tablero
	tableroCopia.insertarFicha(f, posXTablero(pPulsar), posYTablero(pPulsar)); // se deja donde estaba.
    }
    // Ponemos a null el puntero una vez a finalizado su utilidad, para evitar errores.
    f = Ficha(false, 0);
    soltar.reproducir();
  }
  
  return false;
}

bool Partida::eventosTeclado()
{
  bool finTurno = false;

  if (evento.key.keysym.sym == SDLK_m && !pausa) { // mute
    musica.silenciar();
  }

  else if (evento.key.keysym.sym == SDLK_n && !pausa) { // Ordenar por número
    soporteCopia.ordenarNumero();
    jugadores[jugadorActual]->obtenerSoporte() = soporteCopia;
  }

  else if (evento.key.keysym.sym == SDLK_c && !pausa) { // Ordenar por color
    soporteCopia.ordenarColor();
    jugadores[jugadorActual]->obtenerSoporte() = soporteCopia;
  }

  else if (evento.key.keysym.sym == SDLK_r && !pausa) { // Robar
    finTurno  = true;
    robar = true;
  }
  
  else if (evento.key.keysym.sym == SDLK_s && !pausa) { // Comprobar
    finTurno =  true;
    comprobar = true;
  }

  else if (evento.key.keysym.sym == SDLK_p) {
    pausa = !pausa;
    if (pausa) {
      crono.pausa();
      musica.pausa();
    }
    else {
      crono.continuar();
      musica.continuar();
    }
  }

    return finTurno;
}
