#ifndef _PUNTERO_H_
#define _PUNTERO_H_

#include <SDL/SDL.h>

/** Clase Puntero. Clase para el manejo y funcionamiento del puntero del ratón.
*
* Distribuido bajo licencia GNU GPLv.3
*
* @author Daniel Chaves Vázquez
* @version 1.0
*/
class Puntero {
 public:

  // Constructor

  /** Constructor de la clase Puntero.
   */
  Puntero();

  // Operador de asignación

  /** Operador de asignación de la clase Puntero.
   */
  Puntero& operator=(const Puntero& p); 

  // Modificadores

  /** Método que actualiza la información del puntero: coordenadas y botón pulsado.
   */
  void actualizar();

  // Observadores

  /** Método que devuelve la coordenada x del puntero.
   * @return Devuelve la coordenada x del puntero.
   */
  int posX() const;

  /** Método que devuelve la coordenada y del puntero.
   * @return Devuelve la coordenada y del puntero.
   */
  int posY() const;

  /** Método que devuelve el botón con el que se realizó el evento del ratón.
   * @return Devuelve 1 si es el botón izquierdo, 2 si es el botón central y 3 si es el botón derecho.
   */
  int boton() const;

  /** Destructor de la clase Puntero.
   */
  ~Puntero();

 private:
  int x, y;
  int bot;
};

inline Puntero::Puntero(): x(0), y(0), bot(0) {}
inline Puntero& Puntero::operator=(const Puntero& p) { if (this != &p) { x = p.x; y = p.y; bot = p.bot; } return *this; }
  inline void Puntero::actualizar() { SDL_PumpEvents(); bot = SDL_GetMouseState(&x, &y); }
inline int Puntero::posX() const { return x; }
inline int Puntero::posY() const { return y; }
inline int Puntero::boton() const { return bot; }
inline Puntero::~Puntero() {}

#endif

