#ifndef _SONIDO_H_
#define _SONIDO_H_

#include <SDL/SDL.h>
#include <SDL/SDL_mixer.h>
#include "puntero.h"

/** Clase Efecto. Clase para gestionar los efectos. 
*
* Distribuido bajo licencia GNU GPLv.3
*
* @author Daniel Chaves Vázquez
* @version 1.0
*/
class Efecto
{
 public:

  // Constructor

  /** Constructor de la clase Musica
   * @param ruta Ruta del archivo de audio a reproducir (WAV).
   */
  Efecto(const char* ruta);

  // Modificadores

  /** Método para reproducir el efecto.
   * @param loops Si es -1 se reproducirá indefinidamente, si es 0 una sola vez. Por defecto es 0.
   */
  void reproducir(int loops = 0);

  /** Destructor de la clase Efecto.
   */
  ~Efecto();

 private:
  Mix_Chunk *efecto;
};

inline Efecto::~Efecto() { Mix_FreeChunk(efecto); }

/** Clase Musica. Clase para gestionar la musica. 
*
* Distribuido bajo licencia GNU GPLv.3
*
* @author Daniel Chaves Vázquez
* @version 1.0
*/
class Musica {
 public:

  // Constructor

  /** Constructor de la clase Musica
   * @param ruta Ruta del archivo de audio a reproducir (WAV).
   */
  Musica(const char* ruta);

  // Modificadores

  /** Método para silenciar la música (Entrar en modo mute).
   */ 
  void silenciar();

  /** Método para reproducir la música.
   */
  void reproducir();

  /** Método para pausar la música.
   */
  void pausa();

  /** Método para, tras pausar, continuar la música.
   */
  void continuar();

  /** Destructor de la clase Musica.
   */
  ~Musica();

 private:
  Mix_Music *musica;
};

// Funciones externas a la clase

/** Función que devuelve si las coordenadas de un puntero están en el area del altavoz o no.
 * @param p Referencia constante a un objeto de la clase Puntero.
 * @return Devuelve si las coordenadas del puntero están o no en el area del altavoz.
 */
bool areaAltavoz(const Puntero& p);
inline Musica::~Musica() { Mix_FreeMusic(musica); }
#endif
