#include <iostream>
#include "video.h"
#include "constantes.h"
#include "puntero.h"
#include "soporte.h"

using namespace std;

Soporte::Soporte(): fichRest(0), suma(0), sigExt(false, 0), abrto(false) {
  for (int i = 0; i < CAPACIDAD; i++)
    sop[i] = Ficha(false, 0);
}

Soporte::Soporte(const Soporte& s): fichRest(s.fichRest), suma(s.suma), abrto(s.abrto)
{
  for (int i = 0; i < CAPACIDAD; i++)
    sop[i] = s.sop[i];
}

Soporte& Soporte::operator =(const Soporte& s)
{
  if (this != &s)
    {
      for (int i = 0; i < CAPACIDAD; i++)
	sop[i] = s.sop[i];
      
      fichRest = s.fichRest;
      suma = s.suma;
      abrto = s.abrto;
    }
  return *this;
}

void Soporte::mostrarSoporte(Video *v)
{
  int posicionX = SOP_FICH_X, posicionY = SOP_FICH_Y;

  // cout << endl << "MOSTRANDO SOPORTE" << endl << endl;
  if (fichRest != 0) {
    for(int i = 0; i < CAPACIDAD; i++) {
      if (i == 14) {
	posicionY += FICHA_H;
	posicionX = SOP_FICH_X;
      }

      if (!sop[i].fichaNula()) {
	sop[i].posicionar(posicionX, posicionY);
	sop[i].mostrarFicha(v);
      }
      posicionX += FICHA_W;
    }
  }
  //else
  //cout << "el Soporte esta vacio" << endl;
}

const Ficha& Soporte::escogerFicha(int pos)
{
  sigExt = Ficha(false, 0);

  if (!sop[pos].fichaNula()) {
    sigExt = sop[pos];
    sop[pos] = Ficha(false, 0);

    if (sigExt.comodin())
      suma -= 30;
    else
      suma -= sigExt.numero();

    fichRest--;
  }
  
  return sigExt;
}

bool Soporte::ordenar(const Ficha& f, int pos)
{
  if (sop[pos].fichaNula()) {
    sop[pos] = f;

    if (sop[pos].comodin())
      suma += 30;
    else
      suma += sop[pos].numero();

    fichRest++;

    return true;
  }

  else
    return false;
}

void Soporte::robar(Monton& m)
{
  if (!m.vacio() && fichRest < CAPACIDAD) {
    int i = 0;

    while (!sop[i].fichaNula())
      i++;

    sop[i] = m.extraer();

    if (sop[i].comodin())
      suma += 30;
    else
      suma += sop[i].numero();

    fichRest++;
  }
 
  else if (m.vacio())
    cout << "No se puede robar, el monton está vacio\n\n";

  else if (fichRest == CAPACIDAD)
    cout << "No se puede robar, el soporte está lleno\n\n";
}

void Soporte::ordenarNumero()
{
  juntarFichas();

  for (int i = 1; i < fichRest; i++) {
    for (int j = i; j > 0; j--) {
      if (sop[j].comodin() || (sop[j].numero() < sop[j-1].numero() && !sop[j-1].comodin()))
	intercambiar(j, j-1);
    }
  }
}

void Soporte::ordenarColor()
{
  juntarFichas();
  for (int i = 1; i < fichRest; i++) {
    for (int j = i; j > 0; j--) {
      if (sop[j].comodin())
	intercambiar(j, j-1);

      else if (sop[j].color() < sop[j-1].color())
	intercambiar(j, j-1);

      else if (((sop[j].color() == sop[j-1].color() && sop[j].numero() < sop[j-1].numero()) && !sop[j-1].comodin()))
	intercambiar(j, j-1);
    }
  }
}

Soporte::~Soporte() {}

void Soporte::juntarFichas()
{
  bool restan = true; // Indica si quedan fichas en las posiciones del soporte
  int i = 0, j;

  // En primer lugar, juntamos todas las fichas en las primeras posiciones del vector
  while (i < CAPACIDAD && restan) {
    if (sop[i].fichaNula()) { // Si nos encontramos una posición vacia
      j = i;
      while (j < CAPACIDAD && sop[j].fichaNula()) // buscamos una ficha hasta que se acabe el soporte o hasta que encontremos alguna
	j++;

      if (j == CAPACIDAD) // Si j = CAPACIDAD es que hemos llegado al final del soporte sin encontrar ninguna ficha, por tanto ya solo quedan huecos
	restan = false;
      else // Si no, hemos encontrado una ficha, la insertamos en la primera posición vacia
	ordenar(escogerFicha(j), i);
    }
    i++;
  }
}

void Soporte::intercambiar(const int pos1, const int pos2)
{
  Ficha f;

  f = escogerFicha(pos1);
  ordenar(escogerFicha(pos2), pos1);
  ordenar(f, pos2);
}

int posSoporte(const Puntero& p)
{
  int c = (p.posY() / FICHA_H) * FICHA_W + p.posX() / FICHA_W;

  if (c <=470 )
    return c - 457;
  else
    return 14 + c - 489;  
}

bool areaSoporte(const Puntero& p)
{
  return ((p.posX() >= SOP_FICH_X && p.posX() < SOP_FICH_XW) && (p.posY() >= SOP_FICH_Y && p.posY() < SOP_FICH_YH));
}

bool areaColor(const Puntero& p)
{
  return ((p.posX() >= ORD_COL_X && p.posX() < ORD_COL_XW) && (p.posY() >= ORD_COL_Y && p.posY() < ORD_COL_YH));
}

bool areaNumero(const Puntero& p)
{
  return ((p.posX() >= ORD_NUM_X && p.posX() < ORD_NUM_XW) && (p.posY() >= ORD_NUM_Y && p.posY() < ORD_NUM_YH));
}
