#include <iostream>
#include "video.h"
#include "constantes.h"
#include "puntero.h"
#include "tablero.h"

using namespace std;

Tablero::Tablero(): sigExt(false, 0)
{
  for (int i = 0; i < FILAS; i++) {
    for (int j = 0; j < COLUMNAS; j++) {
      tab[i][j] = Ficha(false, 0);
    }
  } 
}

Tablero::Tablero(const Tablero& t): sigExt(t.sigExt)
{
  for (int i = 0; i < FILAS; i++) {
    for (int j = 0; j < COLUMNAS; j++) {
      tab[i][j] = t.tab[i][j];
    }
  } 
}

Tablero& Tablero::operator =(const Tablero& t)
{
  if (this != &t) {
    sigExt = t.sigExt;
    for (int i = 0; i < FILAS; i++) {
      for (int j = 0; j < COLUMNAS; j++) {
	tab[i][j] = t.tab[i][j];
      }
    } 
  }
  return *this;
}

const Ficha& Tablero::escogerFicha(int posX, int posY)
{
  sigExt = tab[posX][posY];
  tab[posX][posY] = Ficha(false, 0);

  return sigExt;
}

bool Tablero::insertarFicha(const Ficha& f, int posX, int posY)
{
  if (tab[posX][posY].fichaNula()) {
    tab[posX][posY] = f;
    return true;
  }
   
  else
    return false;
}

void Tablero::mostrarTablero(Video* v)
{
  int posicionX = TABLERO_X, posicionY = TABLERO_Y;

  // cout << "\n\n MOSTRANDO TABLERO\n\n";
  for (int i = 0; i < FILAS; i++) {
    for (int j = 0; j < COLUMNAS; j++) {
      if (posicionX >= TABLERO_XW) {
	posicionX = TABLERO_X;
	posicionY += FICHA_H;
      }
 
      if (!tab[i][j].fichaNula()) {
	tab[i][j].posicionar(posicionX, posicionY);
	tab[i][j].mostrarFicha(v);
      }

      posicionX += FICHA_W;
    }
  }
}

bool Tablero::comprobar()
{
  bool error = false;
  conj.clear();
  int i = 0, j = 0;

  // Mientras que no se haya recorrido toda la matriz y no se encuentre ningún error
  while (i < FILAS && !error) {
    while (j < COLUMNAS && !error) {
      // Si una posición no está vacia, la analizamos.
      if (!tab[i][j].fichaNula()) {
	// Si no es un comodín, comprobamos que forma parte de un grupo válido.
	if (!tab[i][j].comodin())
	  error = !comprobarFicha(tab[i][j]);

	// Si es un comodín y está en la primera casilla de una fila o es la primera ficha de un conjunto
	else if (tab[i][j].comodin() && (j == 0 || tab[i][j-1].fichaNula())) {
	  // Las dos siguientes casillas deben estar ocupadas o de lo contrario el conjunto no sería válido (mínimo tres fichas)
	  if (!tab[i][j+1].fichaNula() && !tab[i][j+2].fichaNula()) {
	    // Si tienen igual numero, se trata de un conjunto
	    if (tab[i][j+1].numero() == tab[i][j+2].numero()) {
	      tab[i][j].modificarComodin(tab[i][j+1].numero(), tab[i][j+1].color());
	      error = !comprobarFicha(tab[i][j]);
	    }	
	    // Si tienen igual color, se trata de una escalera
	    else if (tab[i][j+1].color() == tab[i][j+2].color()) {
	      tab[i][j].modificarComodin(tab[i][j+1].numero() - 1, tab[i][j+1].color());
	      error = !comprobarFicha(tab[i][j]);
	    }
	    else
	      error = true;
	  }
	  else
	    error = true;
	}

	// Si es un comodín y está en la última casilla de una fila o es la última ficha de un conjunto
	else if (tab[i][j].comodin() && (j == 25 || tab[i][j+1].fichaNula())) {
	  // Las dos casillas anteriores deben estar ocupadas o de lo contrario el conjunto no sería válido (mínimo tres fichas)
	  if (!tab[i][j-1].fichaNula() && !tab[i][j-2].fichaNula()) {
	    // Si tienen igual numero, se trata de un conjunto
	    if (tab[i][j-1].numero() == tab[i][j-2].numero()) {
	      tab[i][j].modificarComodin(tab[i][j-1].numero(), tab[i][j-1].color());
	      error = !comprobarFicha(tab[i][j]);
	    }	
	    // Si tienen igual color, se trata de una escalera
	    else if (tab[i][j-1].color() == tab[i][j-2].color()) {
	      tab[i][j].modificarComodin(tab[i][j-1].numero() + 1, tab[i][j-1].color());
	      error = !comprobarFicha(tab[i][j]);
	    }
	    else
	      error = true;
	  }
	  else
	    error = true;
	}

	// Si es un comodín y no esta ni al principio ni al final de una fila o conjunto
	else if (tab[i][j].comodin()) {
	  // Comprobamos que las casillas anterior y siguiente no estén vacias
	  if (!tab[i][j-1].fichaNula() && !tab[i][j+1].fichaNula()) {
	    // Si tienen igual numero, se trata de un conjunto
	    if (tab[i][j-1].numero() == tab[i][j+1].numero()) {
	      tab[i][j].modificarComodin(tab[i][j-1].numero(), tab[i][j-1].color());
	      error = !comprobarFicha(tab[i][j]);
	    }	
	    // Si tienen igual color, se trata de una escalera
	    else if (tab[i][j-1].color() == tab[i][j+1].color()) {
	      tab[i][j].modificarComodin(tab[i][j-1].numero() + 1, tab[i][j-1].color());
	      error = !comprobarFicha(tab[i][j]);
	    }
	    else
	      error = true;
	  }
	  else
	    error = true;
	}
      }

      // si encontramos una casilla a NULL,
      else {
	// Comprobamos que la longitud del conjunto anterior sea válida, sin estar vacio
	if (conj.size() > 0 && conj.size() < 3)
	  error = true; 
	// Vaciamos el conjunto
	if (!conj.empty())
	  conj.clear();
      }

      j++;
    }
    // Si al llegar al final de la fila el conjunto no es valido, se comete un error.
    if (conj.size() > 0 && conj.size() < 3)
      error = true; 

    // Vaciamos el conjunto
    if (!conj.empty())
      conj.clear();
    j = 0;
    i++;
  }

  return !error;
}

Tablero::~Tablero() {}

bool Tablero::comprobarFicha(const Ficha& f)
{
  bool opValida = false;
  // Si el conjunto está vacio, se puede insertar directamente una ficha en el, sin importar que ficha sea.
  if (conj.empty()) {
    conj.push_back(f);
    opValida = true;
  }

  // Si hay una sola ficha tenemos que comprobar si es valida con la que ya está
  else if (conj.size() == 1) {
    Ficha primera = conj[0];

    // Si las dos son comodines, es inválido.
    if (f.comodin() && primera.comodin()) 
      cout << "Operación invalida" << endl;

    // Si alguna de las dos es comodín, se inserta directamente.
    if (f.comodin() || primera.comodin()) {
      if (f.numero() <= primera.numero())
	conj.insert(conj.begin(), f);
      else
	conj.push_back(f);
      opValida = true;
    }

      // Si no, si tienen igual numero y color distinto o si tienen igual color y la diferencia entre los numeros es 1
    else if ((f.numero() == primera.numero() && f.color() != primera.color()) || (f.color() == primera.color() && ((f.numero()== primera.numero() + 1)))) {
      conj.push_back(f);
      opValida = true;
    }

    else 
      cout << "Operacion invalida" << endl;
  }

  // Si hay más de una ficha hay que comprobar que la ficha corresponde con las ya insertadas.
  else if (conj.size() > 1) {
    Ficha primera = conj[0];
    Ficha ultima = conj[conj.size() - 1];

    if ((f.comodin() && ultima.comodin()) || (f.comodin() && primera.comodin())) 
      cout << "Operacion invalida" << endl;

    else if (f.numero() == primera.numero() && f.numero() == ultima.numero() && !colorRepetido(f)) {
	conj.push_back(f);
	opValida = true;
    }

    else if (f.color() == primera.color() && f.color() == ultima.color() && f.numero()== ultima.numero() + 1) {  
        conj.push_back(f);
        opValida = true;
    }
    else 
      cout << "Operacion invalida" << endl;
  }

  return opValida;
}

bool Tablero::colorRepetido(const Ficha& f)
{
  bool bandera = false;

  if (conj.size() >= 4)
    bandera = true; 

  else if (!f.comodin()) {
    for (unsigned i = 0; i < conj.size(); i++)
      if (conj[i].color() == f.color())
	bandera = true;
  }

  return bandera;
}

bool areaTablero(const Puntero& p)
{
  return ((p.posX() >= TABLERO_X && p.posX() < TABLERO_XW) && (p.posY() >= TABLERO_Y && p.posY() < TABLERO_YH));
}

int posXTablero(const Puntero& p)
{
  return p.posY()/FICHA_H - 1;
}

int posYTablero(const Puntero& p)
{
  return p.posX()/FICHA_W - 3;
}

bool areaComprobar(const Puntero& p)
{
  return ((p.posX() >= COMPROBAR_X && p.posX() < COMPROBAR_XW) && (p.posY() >= COMPROBAR_Y && p.posY() < COMPROBAR_YH));
}
