#ifndef _TABLERO_H_
#define _TABLERO_H_

#include <vector>
#include "ficha.h"

using namespace std;

/** Clase Tablero. Clase para el manejo y funcionamiento del tablero del juego.
*
* Distribuido bajo licencia GNU GPLv.3
*
* @author Daniel Chaves Vázquez
* @version 1.0
*/
class Tablero {
 public:

  // Constructores

  /** Constructor de la clase Tablero.
   */
  Tablero();

  /** Constructor de copia de la clase Tablero.
   * @param t Referencia constante a un objeto de la clase Tablero.
   */
  Tablero(const Tablero& t);

  // Operador de asignacion

  /** Operador de asignación de la clase Tablero.
   * @param t Referencia constante a un objeto de la clase Tablero.
   */
  Tablero& operator =(const Tablero& t);

  // Modificadores

  /** Método para extraer una ficha del Tablero.
   * @param posX Posición en el eje X del tablero.
   * @param posY Posición en el eje Y del tablero.
   * @return Devuelve una referencia constante a un objeto de la clase Ficha. En caso de no haber ficha en una determinada posición, devuelve una ficha nula.
   */
  const Ficha& escogerFicha(int posX, int posY);

  /** Método para insertar una ficha en el Tablero.
   * @param f Referencia constante a un objeto de la clase Ficha.
   * @param posX Posición en el eje X del tablero.
   * @param posY Posición en el eje Y del tablero.
   * @return Devuelve si la ficha se ha podido insertar o no.
   */
  bool insertarFicha(const Ficha& f, int posX, int posY);

  // Observadores

  /** Método encargado de mostrar el tablero por pantalla.
    * @param v Puntero a un objeto de la clase que contiene la superficie donde se van a dibujar las imagenes.
    */
  void mostrarTablero(Video* v);

  /** Método encargado de comprobar la validez de las combinaciones existentes en el tablero.
   * @return Devuelve si el tablero es correcto o no.
   */
  bool comprobar();

  /** Destructor de la clase Tablero.
   */
  ~Tablero();

 private:
  Ficha sigExt;
  Ficha tab[11][26];

  /** Método auxiliar de comprobar, que comprueba si una ficha es válida con las demás que se estén comprobando.
   * @param f Referencia constante a un objeto de la clase Ficha.
   * @return Devuelve si la ficha es válida con las demás que se están comprobando o no.
   */
  bool comprobarFicha(const Ficha& f);

  /** Método auxiliar de comprobarFicha, que comprueba si para un conjunto, el color está repetido o no.
   * @param f Referencia constante a un objeto de la clase Ficha.
   * @return Devuelve si el color de la ficha está repetido en el conjunto que se está comprobando.
   */
  bool colorRepetido(const Ficha& f);

  typedef vector<Ficha> Conjunto;
  Conjunto conj;
};

// Funciones externas

/** Función que devuelve si las coordenadas de un puntero están dentro del area del tablero o no.
 * @param p Referencia constante a un objeto de la clase Puntero.
 * @return Devuelve si las coordenadas del puntero están o no en el area del tablero.
 */
bool areaTablero(const Puntero& p);

/** Función que devuelve la coordenada en el eje X dentro del tablero a partir de las coordenadas del puntero.
 * @param p Referencia constante a un objeto de la clase Puntero.
 * @return Devuelve la coordenada en el eje X dentro del tablero a partir de las coordenadas del puntero. 
 */
int posXTablero(const Puntero& p);

/** Función que devuelve la coordenada en el eje Y dentro del tablero a partir de las coordenadas del puntero.
 * @param p Referencia constante a un objeto de la clase Puntero.
 * @return Devuelve la coordenada en el eje Y dentro del tablero a partir de las coordenadas del puntero. 
 */
int posYTablero(const Puntero& p);

/** Función que devuelve si las coordenadas de un puntero están dentro del area del botón comprobar o no.
 * @param p Referencia constante a un objeto de la clase Puntero.
 * @return Devuelve si las coordenadas del puntero están o no en el area del botón comprobar.
 */
bool areaComprobar(const Puntero& p);

#endif

