#ifndef _VIDEO_H_
#define _VIDEO_H_

#include <SDL/SDL.h>
#include <SDL/SDL_mixer.h>

/** Clase Video. Clase para gestionar el sistema de video.
*
* Distribuido bajo licencia GNU GPLv.3
*
* @author Daniel Chaves Vázquez
* @version 1.0
*/
class Video {
 public:

  // Constructor

  /** Constructor de la clase Video.
   * @param ancho Anchura de la ventana. Por defecto 1024.
   * @param alto	 Altura de la ventana. Por defecto 768.
   * @param prof Profuncidad del color. Por defecto 32
   * @param full Indica si el juego se ejecutará en pantalla completa o no.
   */ 
  Video(int ancho = 1024, int alto = 768, int prof = 32, bool full = false);

  // Observadores

  /** Método que devuelve la superficie de la pantalla.
   * @return Devuelve la pantalla. 
   */
  SDL_Surface* superficie();

  /** Método que dibuja en pantalla lo que hay en el buffer de *pantalla.
   */
  void actualizar();

  /** Método que devuelve el ancho de la pantalla.
   * @return Devuelve el ancho de la pantalla. 
   */
  int anchoSup() const;

  /** Método que devuelve el alto de la pantalla.
   * @return Devuelve el alto de la pantalla. 
   */
  int altoSup() const;

  /** Método que devuelve la profundidad de la pantalla.
   * @return Devuelve la profundidad de la pantalla. 
   */
  int profSup() const;

  /** Destructor de la clase Video-
   */
  ~Video();

 private:
  int  ancho, alto, profundidad;
  SDL_Surface *pantalla;
};

inline void Video::actualizar() { SDL_Flip(pantalla); }
inline SDL_Surface* Video::superficie() { return pantalla; }
inline int Video::anchoSup() const { return ancho; }
inline int Video::altoSup() const { return alto; }
inline int Video::profSup() const { return profundidad; }
inline Video::~Video() { SDL_FreeSurface(pantalla); }

#endif
